#!/usr/bin/perl
#
#----------------------------------------------------------------------
# heading     : Telephony
# description : Global Settings
# navigation  : 4000 4801
#
# Copyright   (c) aelitra.com        2005-2010
#
#----------------------------------------------------------------------
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
#
# Technical support for this program is available from Selintra Ltd
# Please visit our web site www.selintra.com/ for details.
#----------------------------------------------------------------------

use lib '/opt/sark/perl/modules';
use Tie::File;

use strict;
#use CGI:;Fast;
use CGI ':all';
use CGI::Carp qw(fatalsToBrowser);
use sark::SarkSubs;
#use Net::Domain qw(hostname hostfqdn hostdomain);
sub showInitial ($$);
sub updateGlobals ($);
sub performRestore ($);
sub performDump ($);
sub readyRestore ($);
sub doRestore ($);

BEGIN
{
    # Clear PATH and related environment variables so that calls to
    # external programs do not cause results to be tainted. See
    # "perlsec" manual page for details.

    $ENV {'PATH'} = '';
    $ENV {'SHELL'} = '/bin/bash';
    delete $ENV {'ENV'};
}

#esmith::util::setRealToEffective ();
$< = $>;
$( = $);
$CGI::POST_MAX=1024 * 100;  # max 100K posts
$CGI::DISABLE_UPLOADS = 1;  # no uploads

my $q = new CGI;

    my @Verrors;
    my %global;
    my %hashVarUpdBuf = ();
    my %hashVarErrors;
    my %hashVarDefaults = (
        "ABSTIMEOUT" => "3600",
        "ACL" => "YES",
    	"AGENTSTART" => "1001",
        "ASTDLIM" => "|",
		"ALLOWHASHXFER" => "disbled",
		"ATTEMPTRESTART" => "NO",
		"BLINDBUSY" => "",
		"BOUNCEALERT" => "",
		"CALLRECORD1" => "None",
		"CAMPONQONOFF" => "OFF",
		"CAMPONQOPT" => "r,,,30",
		"CFWDANSWER" => "disabled",
		"CFWDEXTRNRULE" => "disabled",
		"CFWDPROGRESS" => "disabled",		
        "CLUSTER" => "OFF",
		"CONFTYPE" => "simple",
		"COSSTART" => "OFF",
		"EDOMAIN" => "",
		"EMAILALERT" => "",
		"EMERGENCY" => "911 999 112",
		"EXTLEN" => "4",
		"FAX" => "",
		"FAXDETECT" => "4",
#		"FOPPASS" => "1234",
		"HAAUTOFAILBACK" => "off",
		"HACLUSTERIP" => "",
        "HAMODE" => "disabled",
		"HAENCRYPT" => "sha1",
		"HAPRINODE" => "",
		"HASECNODE" => "",
		"HASYNCH" => "LOOSE",
		"HAUSECLUSTER" => "NO",
		"INTRINGDELAY" => "0",
		"LOCALIP" => "",
		"LOGOPTS" => "",
		"LTERM" => "YES",
        "MISDNRUN" => "disabled",
        "MONITOROUT" => '/home/sark/monout',
        "MONITORSTAGE" => '/home/sark/monstage',
        "ONBOARDMENU" => "disabled",
		"OPERATOR" => "0",
		"PLAYBEEP" => "NO",
		"PLAYBUSY" => "NO",
		"PLAYCONGESTED" => "NO",
		"PROXY" => "NO",
		"PROXYIGNORE" => "",
		"RECQDITHER" => "2",
		"RECLIMIT" => "1000",
		"RECQSEARCHLIM" => "200",
		"RINGDELAY" => "0",
		"RUNFOP" => "enabled",
		"SIPIAXSTART" => "5000",
		"SIPMULTICAST" => 'disabled',
		"SMSALERT" => "",
		"SMSC" => "",
		"SNO" => "",
		"SPYPASS" => "4444",
#                "STRUNK" => "Hangup",
		"SUPEMAIL" => "admin\@mydomain",
		"SYSOP" => "RINGALL",
		"SYSPASS" => "1111",
		"TFTP" => "disabled",
		"USBRECDISK" => "",
		"VDELAY" => "2",
        "VLIBS" => '/var/log /var/tmp /tmp',
		"VOICEINSTR" => "YES",
		"VOIPMAX" => "3",
        "XMPP" => "disabled",
        "XMPPSERV" => "localhost",
        "ZTP" => "disabled",
        );
    my %hashVarRegexs = (
        "ABSTIMEOUT" => qr/(\d{1,5})/,
        "ASTDLIM" => qr/(\||\,)/,
    	"AADEFAULT" => qr/([\w\.-]{1,32})/,
    	"AGENTSTART" => qr/(\d{3,4})/,
#               "ALERT" => qr/(^\w[-._\w]*\w@\w[-._\w]*\w\.\w{2,3}$)/,
		"CAMPONQOPT" => qr/([\w|,]{1,32})/,
		"EDOMAIN" => qr/(^([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])$)/,
		"EMAILALERT" => qr/(^\w[-\._\w]*\w@\w[-\._\w]*\w\.\w{2,3}$)/,
		"EMERGENCY" => qr/([\d\s]{1,32})/,
		"FAXDETECT" => qr/(\d{1,2})/,
#		"FOPPASS" => qr/(\d{4})/,
		"HACLUSTERIP" => qr/(^([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])$)/,
        "HAPRINODE" => qr/([\w-]{1,32})/,
		"HASECNODE" => qr/([\w-]{1,32})/,
		"INTRINGDELAY" => qr/(\d{1,2})/,
#		"LCLVOIPMAX" => qr/(\d{1,2})/,
		"LOCALIP" => qr/(^([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])$)/,
#		"LOGBAK" => "",
#		"LOGOPTS" => "",
		"OPERATOR" => qr/(\d{1,14})/,
		"PROXYIGNORE" => qr/([\w\s\/-=\?]{1,32})/,
		"RECQDITHER" => qr/(\d)/,
		"RECLIMIT" => qr/(\d)/,
		"RECQSEARCHLIM" => qr/(\d{2,4})/,
		"RINGDELAY" => qr/(\d{1,2})/,
		"SIPIAXSTART" => qr/(\d{3,4})/,
		"SMSALERT" => qr/([\d ]{1,32})/,
#		"SMSC" => "",
		"SPYPASS" => qr/(\d{4})/,
		"SUPEMAIL" => qr/(^\w[-\._\w]*\w@\w[-\._\w]*\w\.\w{2,3}$)/,
#		"SYSOP" => qr/(\d{1,4})/,
		"SYSPASS" => qr/(\d{4})/,
		"VOIPMAX" => qr/(\d{1,3})/,
        "XMPPSERV" => qr/([\d\.\@\w_]+)/,
        );

my %hashColorDefaults = %hashVarDefaults;
while ( my $tk = each %hashColorDefaults ) {
        $hashColorDefaults{$tk} = 'black';
}


my $buttoncolour = 'lightskyblue';
my $maxundonum = '5';
#hpealias
#my $nrdigitsColor = 'black';
my $commit;
#my $page;
my $sno;
#my $undo;
my $warp;
my $warpsdcard;
#my @disksize;
#my @diskfields;
#my $maxrecallowed = 1000;


if ( -e "/persistent" ) {
	$warp = 1;
	$_ = `/bin/df`;
	if ( /mmcblk0p1/ ) {
		$warpsdcard = 1;
#		@disksize = `df -k $dir`;
#		@diskfields = split /\s+/, $disksize[1];
#		print "disk size is $fields[1] \n";
#		$maxrecallowed = $fields[1] * .65;		
	}
}

if ($q->param ('restore') eq SarkSubs::tt_display('SarkbuttonRestore'))
{
        performRestore ($q);
}

elsif ($q->param ('start.x'))
{
	performStopStart ($q, "START");
}

elsif ($q->param ('stop.x'))
{
	performStopStart ($q, "STOP");
}

elsif ($q->param ('ha_start.x'))
{
	performStopStart ($q, "HA ON");
}

elsif ($q->param ('ha_stop.x'))
{
	performStopStart ($q, "HAOFF");
}

elsif ($q->param ('commit.x'))
{
        updateGlobals ($q);
}

elsif ($q->param ('undo.x'))
{
    SarkSubs::SQLiteUndo();
    $q->delete_all();
    showInitial ($q,"Undo Processing complete");
}

elsif (! grep (/^state$/, $q->param)) #|| $q->param ('back') eq SarkSubs::tt_display('SarkbuttonBack'))
{
    showInitial ($q,"");
}

elsif ($q->param ('state') eq "updateGlobals")
{
    updateGlobals ($q);
}

elsif ($q->param ('state') eq 'readyrestore')
{
    readyRestore ($q);
}

elsif ($q->param ('state') eq 'doRestore')
{
    doRestore ($q);
}


else
{
#    esmith::cgi::genStateError ($q, \%conf);
     die "STATE ERROR!!  - ending \n";
}

exit (0);

#------------------------------------------------------------
# subroutine to display initial form
#------------------------------------------------------------

sub showInitial ($$)
{
    my ($q,$response) = @_;
    my @rev = `/bin/rpm -q sail`;
    my @free = `/usr/bin/free`;

    my $dbh = SarkSubs::SQLiteConnect();
    my %global = SarkSubs::SQLiteGetHash($dbh, "SELECT * FROM globals WHERE pkey = 'global'");
    SarkSubs::SQLiteDisconnect($dbh);
    $commit = $global{MYCOMMIT};

    my $localIP	   = SarkSubs::ret_localip();
    my $systemName = `/bin/hostname`;

    my $harness			= 'PBX';
    my $sipiaxstart    		= ($global{'SIPIAXSTART'});
    $sno			= ($global{'SNO'}) 		|| int(rand(1000000));
    my $misdnrun                = ($global{'MISDNRUN'})		|| 'disabled';
    my $syspass    		= ($global{'SYSPASS'})     	|| '1111';
#    my $foppass    		= ($global{'FOPPASS'})     	|| '1234';
    my $spypass    		= ($global{'SPYPASS'})     	|| '4444';
    my $operator		= ($global{'OPERATOR'})       	|| '0';
    my $sysop			= ($global{'SYSOP'})         	|| '';
    my $edomain   		= ($global{'EDOMAIN'})     	|| '';
    my $countrycode			= ($global{'COUNTRYCODE'})     	|| 'uk';
#    my $compression		= ($global{'COMPRESSION'})    	|| 'LOW';
    my $voipmax			= ($global{'VOIPMAX'})       	|| '3';
#    my $lclvoipmax		= ($global{'LCLVOIPMAX'})    	|| '24';
    my $fax			= ($global{'FAX'})    	 	|| '101';
    my $faxdetect		= ($global{'FAXDETECT'})     	|| '2';
    my $ringdelay		= ($global{'RINGDELAY'})     	|| '0';
    my $intringdelay		= ($global{'INTRINGDELAY'})  	|| '10';
    my $agentstart		= ($global{'AGENTSTART'})	|| '1001';
#    my $cdr		       	= ($global{'CDR'})	     	|| 'NO';
    my $runfop		       	= ($global{'RUNFOP'})	     	|| 'disabled';
#    my $skin		       	= ($global{'SKIN'})	     	|| 'SME';
    my $callrecord1		= ($global{'CALLRECORD1'})	|| 'None';
#    my $callrecord2		= ($global{'CALLRECORD2'})	|| 'None';
    my $tftp			= ($global{'TFTP'})		|| 'disabled';
    my $voiceinstr		= ($global{'VOICEINSTR'})	|| 'YES';
    my $proxy			= ($global{'PROXY'})		|| 'NO';
    my $proxyignore		= ($global{'PROXYIGNORE'})	|| '';
    my $lterm			= ($global{'LTERM'})     	|| 'YES';
    my $supemail		= ($global{'SUPEMAIL'})     	|| 'admin@yourco.com';
    my $emergency               = ($global{'EMERGENCY'})     	|| '911 999 112';
#    my $nrdigits		= ($global{'DIGITS'})		|| 'None';
    my $logopts			= ($global{'LOGOPTS'})		|| 'native';
    my $playbusy		= ($global{'PLAYBUSY'})		|| 'NO';
    my $playcongested		= ($global{'PLAYCONGESTED'})	|| 'NO';
    my $playbeep		= ($global{'PLAYBEEP'})		|| 'NO';
    my $conftype		= ($global{'CONFTYPE'})		|| 'simple';
#    my $logbak			= ($global{'LOGBAK'})		|| 'default';
#    my $logrotate		= ($global{'LOGROTATE'})	|| 'default';
    my $alert			= ($global{'ALERT'})   		|| 'None';
    my $smsc                    = ($global{'SMSC'})   		|| '';
    my $smsalert		= ($global{'SMSALERT'})      	|| '';
    my $emailalert		= ($global{'EMAILALERT'})      	|| '';
#    my $grptransform		= ($global{'GRPTRANSFORM'})    	|| '';
#    my $grpclid			= ($global{'GRPCLID'})      	|| '';
    my $cosstart		= ($global{'COSSTART'})      	|| '';
    my $haprinode		= ($global{'HAPRINODE'}) 	|| $systemName;
    my $hasecnode	       	= ($global{'HASECNODE'}) 	|| '';
    my $haclusterip    		= ($global{'HACLUSTERIP'})     	|| $localIP;
    my $hamode    		= ($global{'HAMODE'})     	|| 'OFF';
    my $hasynch    		= ($global{'HASYNCH'})     	|| 'LOCK';
    my $haencrypt		= ($global{'HAENCRYPT'})    	|| '';
    my $haautofailback		= ($global{'HAAUTOFAILBACK'})  	|| 'on';
    my $hausecluster           	= ($global{'HAUSECLUSTER'})  	|| 'NO';
    my $bouncealert		= ($global{'BOUNCEALERT'})    	|| '';
    my $camponqopt		= ($global{'CAMPONQOPT'})    	|| 'r,,,30';
    my $camponqonoff		= ($global{'CAMPONQONOFF'})    	|| 'OFF';
    my $blindbusy		= ($global{'BLINDBUSY'})    	|| '';
    my $extlen			= ($global{'EXTLEN'})    	|| '4';
#    my $mtime			= ($global{'MTIME'})    	|| 'disabled';
#    my $allowhashxfer		= ($global{'ALLOWHASHXFER'})   	|| 'disabled';
    my $attemptrestart		= ($global{'ATTEMPTRESTART'})   || 'NO';
    my $recqdither		= ($global{'RECQDITHER'})   	|| '2';
    my $recqitdelay		= ($global{'RECQITDELAY'})   	|| '30';
    my $recqsearchlim		= ($global{'RECQSEARCHLIM'})    || '200';
    my $abstimeout		= ($global{'ABSTIMEOUT'})    	|| '3600';
    my $usbrecdisk 		= ($global{'USBRECDISK'})       || '';
    my $astdlim 		= ($global{'ASTDLIM'})       	|| '|';
    my $xmpp	 		= ($global{'XMPP'})       	|| 'disabled';
    my $xmppserv 		= ($global{'XMPPSERV'})       	|| 'localhost';
#    my $strunk 	      		= ($global{'STRUNK'})       	|| 'Hangup';
    my $cluster 	      	= ($global{'CLUSTER'})       	|| 'OFF';
    my $sipmulticast 	= ($global{'SIPMULTICAST'})       	|| 'disabled';
    my $ztp				= ($global{'ZTP'})       	|| 'disabled';
    my $acl				= ($global{'ACL'})       	|| 'YES';
    my $cfwdanswer		= ($global{'CFWDANSWER'})   || 'disabled';
    my $cfwdextrnrule	= ($global{'CFWDEXTRNRULE'})  || 'disabled';
 	my $cfwdprogress	= ($global{'CFWDPROGRESS'})   || 'disabled';

    unless ($extlen) {
    	$extlen = "4";
    }

#   unless ($sipiaxstart) {
#   	if ($extlen  eq "4") {
#       	$sipiaxstart = '5001';
#               $sysop = '5001';
#       }
#       else {
#       	$sipiaxstart = '501';
#               $sysop = '501';
#       }
#   }
#
#
#
    my $astate = "STOPPED";
    my $hstate = "STOPPED";
    my $rstate = "PRIMARY";

    if (SarkSubs::check_pid) {
    	$astate = "RUNNING";
    }
    if ( -e "/usr/lib/heartbeat/heartbeat" ) {
    	if (SarkSubs::check_hapid) {
    		$hstate = "RUNNING";
    	}
    }
    else {
    	$hstate = "Not Installed";
    }
#    if (($global{'HASECNODE'}) eq `/sbin/e-smith/config gettype SystemName`)  {
#    	$rstate = "STANDBY";
#    }
#


    my @Conferences = ();
    my @Vmails = ();
    my @Siblings = ();
    $dbh = SarkSubs::SQLiteConnect();
    my @CustomApps = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM Appl" ));
    my @Clusters = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM Cluster"));
    my @Lines = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM lineIO" ));
    my @Extensions = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM IPphone" ));
    my @Carriers = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM Carrier"));
    my @Queues = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM Queue" ));
    my @SpeedDials = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM speed"));
    SarkSubs::SQLiteDisconnect($dbh);

    foreach (@Extensions)  {
        push (@Vmails, "*".$_);
    }
    foreach (@Lines) {
        push (@Siblings, $_)
        	if (/\~/);
    }
    
    my @indications;
    my $S32ntpd; 
    my $ntpconf;
    my $localenv;  
#    my $linuxtz; 
    
#
#   set timezone for warp (not done here anymore - see ntp)
#
#    if ($warp) {
#		$S32ntpd = `/bin/grep '#sarkSetValue' /persistent/autorun/S32ntpd`; 
#		$S32ntpd =~ /^\s*export\s*TZ=([^\s]+)\s*#sarkSetValue$/;   
#		$linuxtz = $1 || 'GMT0BST';
#   }

    open(FILE,"/etc/asterisk/indications.conf");
    while (<FILE>) {
    	push (@indications, $_);
    }
    close(FILE);
    my @CountryCode;
    my $ccode;
    foreach (@indications) {
		if (/\[(\w\w)\]/) {
			push (@CountryCode, $1);
		}
		if ( /country=(\w\w)/ ) {
				$ccode = $1;
		}
	}
		
	

#DEBUG CODE
#    foreach (@Verrors) {
#    	print STDERR "TOP-VERROR IS $_ \n";
#    }
#DEBUG CODE/
    my $systate = "FULL";
    if ( -e "/opt/sark/.modeone") {
    	$systate = "DEMO";
    }
    SarkSubs::sark_header ($q);

    print h1( SarkSubs::tt_display('Sarksystemh11').($response || "Globals.") );
    print "<hr class=\"hr\">";
    Global_action();

#
#    style="width:500px; background:#dddddd;"

#    print '<div>';
    print '<div id="pagetabs"  class="ui-tabs" >';

    print '<ul>'."\n";
    print '<li><a href="#system">General</a></li>'."\n";
    print '<li><a href="#services" >Services</a></li>'. "\n";
    print '<li><a href="#control" >Call-Control</a></li>'."\n";
    print '<li><a href="#sysadmin" >Admin</a></li>'."\n";
    if ( -e "/etc/init.d/heartbeat" ) {
        print '<li><a href="#ha" >High-Availability</a></li>'."\n";
    }
#    print '<li><a href="#sysinfo" >SysInfo</a></li>'."\n";
    print '</ul>'."\n";



      my $tt_help;
      my $displayname;

    if (@Verrors) {
	SarkSubs::sark_error(\@Verrors);
    }
#    unless ($ENV{HTTP_USER_AGENT} =~ /(MSIE)/ ) {
	printSysNotes();
#    }


#
#       TAB Admin
#
      print '<div id="sysadmin" class="ui-tabs-hide">';

	  unless ( $warp ) {
		print SarkSubs::aLabelFor('astdlim');
		print $q->popup_menu (
                        -name     => 'ASTDLIM',
                        -values   => ['|',','],
                        -default  => $astdlim);
      }
      print SarkSubs::aLabelFor('emergency');
      print $q->textfield (
                	-name     => 'EMERGENCY',
                	-override => 1,
                	-default  => $emergency,
                        -style 	  => "Color: $hashColorDefaults{EMERGENCY}",
        		-size     => 12);
      print SarkSubs::aLabelFor('syspass');
      print $q->textfield (
                	-name     => 'SYSPASS',
                	-override => 1,
                	-default  => $syspass,
                        -style 	  => "Color: $hashColorDefaults{SYSPASS}",
        		-size     => 4);

      print SarkSubs::aLabelFor('spypass');
      print $q->textfield (
                	-name     => 'SPYPASS',
                	-override => 1,
                	-default  => $spypass,
                        -style 	  => "Color: $hashColorDefaults{SPYPASS}",
        		-size     => 4);

#      print SarkSubs::aLabelFor('attemptrestart');
#      print $q->popup_menu (
#                        -name     => 'ATTEMPTRESTART',
#                        -values   => ['NO','YES'],
#                        -default  => $attemptrestart);
#
      print SarkSubs::aLabelFor('supemail');
      print $q->textfield (
                        -name     => 'SUPEMAIL',
                        -override => 1,
                        -default  => $supemail,
                        -style 	  => "Color: $hashColorDefaults{SUPEMAIL}",
                        -size     => 20);

      print SarkSubs::aLabelFor('alert');
      print $q->popup_menu (
                        -name     => 'ALERT',
                        -values   => ['None','sms','email','both'],
                        -default  => $alert);

      print SarkSubs::aLabelFor('smsc');
      print $q->textfield (
                        -name     => 'SMSC',
                        -override => 1,
                        -default  => $smsc,
                        -style 	  => "Color: $hashColorDefaults{SMSC}",
                        -size     => 20);

      print SarkSubs::aLabelFor('smsalert');
      print $q->textfield (
                        -name     => 'SMSALERT',
                        -override => 1,
                        -default  => $smsalert,
                        -style 	  => "Color: $hashColorDefaults{SMSALERT}",
                        -size     => 20);

#      print SarkSubs::aLabelFor('supemail');
#      print $q->textfield (
#                        -name     => 'EMAILALERT',
#                        -override => 1,
#                        -default  => $emailalert,
#                        -style 	  => "Color: $hashColorDefaults{EMAILALERT}",
#                        -size     => 20);
#
#       TAB DIVEND
#
   print "</div>";

#
#       TAB HA
#
      print '<div id="ha" class="ui-tabs-hide">';

      print SarkSubs::aLabelFor('hasynch');
      print $q->popup_menu (
                        -name     => 'HASYNCH',
                        -values   => ['LOOSE','LAZY'],
                        -default  => $hasynch);

      print SarkSubs::aLabelFor('haclusterip');
      print $q->textfield (
                        -name     => 'HACLUSTERIP',
                        -override => 1,
                        -default  => $haclusterip,
                        -style 	  => "Color: $hashColorDefaults{SMSC}",
                        -size     => 20);

      print SarkSubs::aLabelFor('haprinode');
      print $q->textfield (
                        -name     => 'HAPRINODE',
                        -override => 1,
                        -default  => $haprinode,
                        -style 	  => "Color: $hashColorDefaults{SMSALERT}",
                        -size     => 20);

      print SarkSubs::aLabelFor('hasecnode');
      print $q->textfield (
                        -name     => 'HASECNODE',
                        -override => 1,
                        -default  => $hasecnode,
                        -style 	  => "Color: $hashColorDefaults{SMSALERT}",
                        -size     => 20);

      print SarkSubs::aLabelFor('hausecluster');
      print $q->popup_menu (
                        -name     => 'HAUSECLUSTER',
                        -values   => ['NO','YES'],
                        -default  => $hausecluster);

      print SarkSubs::aLabelFor('haautofailback');
      print $q->popup_menu (
                        -name     => 'HAAUTOFAILBACK',
                        -values   => ['on','off'],
                        -default  => $haautofailback);
#
#       TAB DIVEND
#
   print "</div>";

#
#       TAB Control
#
      print '<div id="control" class="ui-tabs-hide">';

#      Global_action();
      print SarkSubs::aLabelFor('playcongested');
      print $q->popup_menu (
                	-name     => 'PLAYCONGESTED',
                	-values   => ['YES','NO','SIGNAL'],
                	-default  => $playcongested);

      print SarkSubs::aLabelFor('playbusy');
      print $q->popup_menu  (
                        -name     => 'PLAYBUSY',
                        -values   => ['YES','NO','SIGNAL'],
                        -default  => $playbusy);

      print SarkSubs::aLabelFor('playbeep');
      print $q->popup_menu (
                	-name     => 'PLAYBEEP',
                	-values   => ['YES','NO'],
                	-default  => $playbeep);
            print SarkSubs::aLabelFor('intringdelay');
      print $q->textfield (
                	-name     => 'INTRINGDELAY',
                	-override => 1,
                	-default  => $intringdelay,
                        -style 	  => "Color: $hashColorDefaults{INTRINGDELAY}",
        		-size     => 4);

      print SarkSubs::aLabelFor('abstimeout');
      print $q->textfield (
                	-name     => 'ABSTIMEOUT',
                	-override => 1,
                	-default  => $abstimeout,
                        -style 	  => "Color: $hashColorDefaults{ABSTIMEOUT}",
        		-size     => 4);
        		
      print SarkSubs::aLabelFor('voipmax');
      print $q->textfield (
                        -name     => 'VOIPMAX',
                        -override => 1,
                        -default  => $voipmax,
                        -style 	  => "Color: $hashColorDefaults{VOIPMAX}",
                        -size     => 4);

      print SarkSubs::aLabelFor('vringdelay');
      print $q->popup_menu (
                	-name     => 'RINGDELAY',
                	-values   => ['0','1','2','3','4','5','6','7','8','9','10'],
                	-default  => $ringdelay);
                	
      print SarkSubs::aLabelFor('cfwdanswer');
      print $q->popup_menu (
					-name    => 'CFWDANSWER',
                  	-values  => ['enabled','disabled'],
                  	-default => $cfwdanswer);       
                  	                	
      print SarkSubs::aLabelFor('cfwdextrnrule');
      print $q->popup_menu (
					-name    => 'CFWDEXTRNRULE',
                  	-values  => ['enabled','disabled'],
                  	-default => $cfwdextrnrule);                       	

      print SarkSubs::aLabelFor('cfwdprogress');
      print $q->popup_menu (
					-name    => 'CFWDPROGRESS',
                  	-values  => ['enabled','disabled'],
                  	-default => $cfwdprogress);       
                  	                
      print SarkSubs::aLabelFor('fax');
      print $q->textfield (
                        -name     => 'FAX',
                        -override => 1,
                        -default  => $fax,
                        -style 	  => "Color: $hashColorDefaults{FAX}",
                        -size     => 4);

      print SarkSubs::aLabelFor('faxdetect');
      print $q->popup_menu (
                        -name     => 'FAXDETECT',
                        -values   => [1,2,3,4,5,6,7,8,9,],
                        -default  => $faxdetect);

#
#       TAB DIVEND
#
   print "</div>";


#
#       TAB Services
#
      print '<div id="services" class="ui-tabs-hide">';

#      Global_action();

      print SarkSubs::aLabelFor('tftp');
      print $q->popup_menu (
			-name    => 'TFTP',
                  	-values  => ['enabled','disabled'],
                  	-default => $tftp);
                  	
      print SarkSubs::aLabelFor('sipmulticast');
      print $q->popup_menu (
			-name    => 'SIPMULTICAST',
                  	-values  => ['enabled','disabled'],
                  	-default => $sipmulticast);
                  	
      print SarkSubs::aLabelFor('ztp');
      print $q->popup_menu (
			-name    => 'ZTP',
                  	-values  => ['enabled','disabled'],
                  	-default => $ztp);                              	

#	  unless ( $warp ) { 
#		print SarkSubs::aLabelFor('xmpp');
#		print $q->popup_menu (
#			-name    => 'XMPP',
#                 	-values  => ['enabled','disabled'],
#                	-default => $xmpp);
#
#		print SarkSubs::aLabelFor('xmppserv');
#        print $q->textfield (
#                	-name     => 'XMPPSERV',
#                	-default  => $xmppserv,
#                	-override => 1,
#                        -style 	  => "Color: $hashColorDefaults{XMPPSERV}",
#        		-size     => 12);
#      } 

#      print SarkSubs::aLabelFor('runfop');
#      print $q->popup_menu (
#			-name    => 'RUNFOP',
#                  	-values  => ['enabled','disabled'],
#                  	-default => $runfop);
#
      if ( $ENV{SERVER_SOFTWARE} =~ /Apache/ ) {
      	print SarkSubs::aLabelFor('proxy');
      	print $q->popup_menu (
                        -name    => 'PROXY',
                        -values  => ['NO','YES'],
                        -default => $proxy);
      }

#     	print SarkSubs::aLabelFor('proxyignore');
#      	print $q->textfield (
#                        -name     => 'PROXYIGNORE',
#                        -override => 1,
#                        -default  => $proxyignore,
#                        -style 	  => "Color: $hashColorDefaults{PROXYIGNORE}",
#                        -size     => 20);
#      }

      print SarkSubs::aLabelFor('cosstart');
      print $q->popup_menu (
                        -name     => 'COSSTART',
                	-values   => ['OFF','ON'],
                	-default  => $cosstart);

      print SarkSubs::aLabelFor('clusterstart');
      print $q->popup_menu (
                        -name     => 'CLUSTER',
                	-values   => ['OFF','ON'],
                	-default  => $cluster);


   if ( -e "/usr/sbin/mISDN") {
      print SarkSubs::aLabelFor('misdnrun');
      print $q->popup_menu (
				-name    => 'MISDNRUN',
                  		-values  => ['enabled','disabled'],
                  		-default => $misdnrun );
   }
   else {
      $misdnrun = 'disabled';
   }
   print SarkSubs::aLabelFor('bouncealert');
   print $q->textfield (
                	-name     => 'BOUNCEALERT',
                	-override => 1,
                	-default  => $bouncealert,
                        -style 	  => "Color: $hashColorDefaults{BOUNCEALERT}",
        		-size     => 20);

   print SarkSubs::aLabelFor('blindbusy');
   print $q->popup_menu (
		  	-name    => 'BLINDBUSY',
                  	-values  => ["Operator", "Hangup",
                               optgroup (-name => "QUEUES",
                                         -values => [@Queues]),
			       optgroup (-name => "EXTENSIONS",
					 -values => [@Extensions]),
			       optgroup (-name => "ALIASES",
					 -values => [@SpeedDials]),
                               optgroup (-name => "LEAVE VOICEMAIL",
                                         -values => [@Vmails]),
			       optgroup (-name => "Custom Apps",
					 -values => [@CustomApps])],
                  	-default => $blindbusy);
   print SarkSubs::aLabelFor('camponqonoff');
   print $q->popup_menu  (
                        -name     => 'CAMPONQONOFF',
                	-values   => ['OFF','ON'],
                	-default  => $camponqonoff);
   print SarkSubs::aLabelFor('camponqopt');
   print  $q->textfield (
                	-name     => 'CAMPONQOPT',
                	-override => 1,
                	-default  => $camponqopt,
                        -style 	  => "Color: $hashColorDefaults{CAMPONQOPT}",
        		-size     => 5);
#
#       TAB DIVEND
#
   print "</div>";

#
#       TAB System  style="background:#cccccc;"
#
      print '<div  id="system" class="ui-tabs-hide">';

#      SarkSubs::printSysNotes();
#      Global_action();
      print SarkSubs::aLabelFor('edomain');
      print $q->textfield (
                	-name     => 'EDOMAIN',
                	-override => 1,,
                	-default  => $edomain,
                    -style 	  => "Color: $hashColorDefaults{EDOMAIN}",
					-size     => 16);
					
      print SarkSubs::aLabelFor('countrycode');
      print $q->popup_menu (
					-name    => 'COUNTRYCODE',
                  	-values  => [@CountryCode],
                  	-default => $countrycode);  
      
#      if ( $warp ) { 
#		print SarkSubs::aLabelFor('linuxtz');            	 
#		print $q->textfield (
#              		name     => 'LINUXTZ',
#              		-override => 1,,
#             		-default  => $linuxtz,
#                   -style 	  => "Color: $hashColorDefaults{LINUXTZ}",
#					-size     => 16);   
#	  }
	   
      print SarkSubs::aLabelFor('voiceinstr');
      print $q->popup_menu (
					-name    => 'VOICEINSTR',
#                   -class    => "mytxt",
                  	-values  => ['YES','NO'],
                  	-default => $voiceinstr);

      print SarkSubs::aLabelFor('lterm');
      print $q->popup_menu (
			-name    => 'LTERM',
                  	-values  => ['YES','NO'],
                  	-default => $lterm);
	  unless ( $warp ) { 
		print SarkSubs::aLabelFor('conftype');
		print $q->popup_menu  (
                        -name     => 'CONFTYPE',
                        -values   => ['simple','hosted'],
                        -default  => $conftype);
      }          

      print SarkSubs::aLabelFor('extlen');
      unless (@Extensions) {
		print $q->popup_menu (
                	-name     => 'EXTLEN',
                	-values  => ['3','4'],
                	-default  => $extlen,
                        -style    => "Color: $hashColorDefaults{EXTLEN}",
        		-size     => 1);
      }
      else {
        print $q->textfield (
                	-name     => 'EXTLEN',
                	-default  => $extlen,
                        -readonly =>'readonly',
                        -style    =>'background-color: lightgrey',
        		-size     => 1);
      }
      
      print SarkSubs::aLabelFor('acl');
      print $q->popup_menu (
			-name    => 'ACL',
                  	-values  => ['YES','NO'],
                  	-default => $acl);              

      print SarkSubs::aLabelFor('sipiaxstart');
      print $q->textfield (
                	-name     => 'SIPIAXSTART',
                	-override => 1,
                	-default  => $sipiaxstart,
                        -style 	  => "Color: $hashColorDefaults{SIPIAXSTART}",
        		-size     => 4);

      print SarkSubs::aLabelFor('agentstart');
      print $q->textfield (
                	-name     => 'AGENTSTART',
                	-override => 1,
                	-default  => $agentstart,
                        -style 	  => "Color: $hashColorDefaults{AGENTSTART}",
        		-size     => 4);
                	
	  if ($warp) {
		  if ($warpsdcard) { 	
			print SarkSubs::aLabelFor('callrecord1');
			print $q->popup_menu (
			-name    => 'CALLRECORD1',
                  	-values  => [SarkSubs::get_rec_list,"Inbound","Outbound","Both"],
                  	-default => $callrecord1);
         }
      }
      else {
      print SarkSubs::aLabelFor('callrecord1');
      print $q->popup_menu (
			-name    => 'CALLRECORD1',
                  	-values  => [SarkSubs::get_rec_list,"Inbound","Outbound","Both"],
                  	-default => $callrecord1);
      }		  
                  	        		

      print SarkSubs::aLabelFor('operator');
      print $q->textfield (
                	-name     => 'OPERATOR',
                	-override => 1,
                	-default  => $global{OPERATOR},
                        -style 	  => "Color: $hashColorDefaults{OPERATOR}",
        		-size     => 4);

      print SarkSubs::aLabelFor('sysop');
      print $q->popup_menu (
                        -name    => 'SYSOP',
                        -values  => [@Extensions,@SpeedDials],
                        -default => $global{SYSOP});

#
#       TAB DIVEND
#
   print "</div>";
#
#       TAB System
#
#      print '<div id="sysinfo">';
#
#      SarkSubs::printSysNotes();
#
#       TAB DIVEND
#
#   print "</div>";

#
#  end of TABS DIV
#
   print "</div>";
   print $q->hidden (-name => 'state', -override => 1, -default => 'updateGlobals');

   SarkSubs::sark_footer($q);
}



sub updateGlobals ($)

{

    my ($q) = @_;
    my %hashScrnVars = $q->Vars ;
    my $response;
    my $gonogo = 0;
    my $dbh = SarkSubs::SQLiteConnect();
    my %global = SarkSubs::SQLiteGetHash($dbh, "SELECT * FROM globals WHERE pkey = 'global'");
    $hashScrnVars{pkey} = 'global';

    $gonogo = SarkSubs::validateTuple(\%hashScrnVars, \%global, \%hashVarRegexs,
    			\%hashVarUpdBuf, \%hashColorDefaults,
                        \@Verrors);
    unless ( $gonogo ) {
    	SarkSubs::updateTuple (\%hashVarUpdBuf, \%global, "globals", $dbh);
    }

    SarkSubs::SQLiteDisconnect($dbh);
#
#   set the country code in indications
#   
	my $rc;
#	$rc = `/bin/sed -i 's/country=\\w\\w/country=$global{COUNTRYCODE}/' /etc/asterisk/indications.conf`;	
# removed request_syscmd sed (because it damages ownership) - r1690 - resurrected -1740
	$rc = SarkSubs::request_syscmd ("/bin/sed -i 's/country=\\w\\w/country=$hashScrnVars{COUNTRYCODE}/' /etc/asterisk/indications.conf");


#
#   if we are on WARP then set the timezones and time server and enable/disable multicast
#
	my $S32ntpd;	
    if ($warp) {
#		$S32ntpd = `/bin/grep '#sarkSetValue' /persistent/autorun/S32ntpd`;
#		$hashScrnVars{"LINUXTZ"} =~ s/^\s+//;
#		$hashScrnVars{"LINUXTZ"} =~ s/\s+$//;
		
#		$S32ntpd =~ /^\s*export\s+TZ=([^\s]+)\s+#sarkSetValue\s*$/;   	
	#
	#	remove the existing time settings 
	#
#		unless ( $hashScrnVars{"LINUXTZ"} eq  $1 ) {
#			SarkSubs::request_syscmd ( "/bin/sed -i '/#sarkSetValue/d' /persistent/autorun/S32ntpd" );
#			SarkSubs::request_syscmd ( "/bin/sed -i '/#sarkSetValue/d' /persistent/etc/localenv" );
	#
	#	add the new/changed ones
	#	
#			SarkSubs::request_syscmd ( "/bin/echo 'export TZ=$hashScrnVars{LINUXTZ} #sarkSetValue' >> /persistent/autorun/S32ntpd" );
#			SarkSubs::request_syscmd ( "/bin/echo 'export TZ=$hashScrnVars{LINUXTZ} #sarkSetValue' >> /persistent/etc/localenv" ); 
#		}
	#
	#	start/stop multicast listener
	#
		if ($hashScrnVars{"SIPMULTICAST"} eq "enabled" ) {
				SarkSubs::request_syscmd ( "/sbin/svc -u  /service/srkmcast" );
				SarkSubs::request_syscmd ( "/bin/rm -f  /service/srkmcast/down" ); 
		}
		else {
				SarkSubs::request_syscmd ( "/sbin/svc -d  /service/srkmcast" );
				SarkSubs::request_syscmd ( "/bin/touch  /service/srkmcast/down" );
		}
    }
    
    if (exists $hashScrnVars{"TFTP"}) {
       	unless ( $hashScrnVars{"TFTP"} eq $global{"TFTP"} ) {
#    		if (-e '/etc/xinetd.d/tftp')  {
    			if ($hashScrnVars{"TFTP"} eq "enabled") {
                        	if ( -e "/sbin/e-smith/config" ) {
    				     $rc =(SarkSubs::request_syscmd ("/sbin/e-smith/config setprop tftp status enabled"));
                                }
                                else {
                                     # chkconfig code goes here
                                }
    				if (! -e '/var/lock/subsys/in.tftpd') {
    				     $rc =(SarkSubs::request_syscmd ("/etc/init.d/tftp start"));
    				}
    			}
    			elsif ($hashScrnVars{"TFTP"} eq "disabled")  {
                        	if ( -e "/sbin/e-smith/config" ) {
    				     $rc =(SarkSubs::request_syscmd ("/sbin/e-smith/config setprop tftp status disabled"));
                                }
                                else {
                                     # chkconfig code goes here
                                }
    				if ( -e '/var/lock/subsys/in.tftpd') {
    					$rc =(SarkSubs::request_syscmd ("/etc/init.d/tftp stop"));
                               	}
                        }
#                }
    	}
    }

    if ($q->param ('commit.x')) {
        SarkSubs::sysCommit($q);
        $response = "Committed";
    }
    else {
        $response = "Saved!";
    }

# send a sighup to the multicast listener if it is running    
    $_ = `/bin/ps ax | /bin/grep responder | /bin/grep -v grep`;
    if ($_) {
	/^\s*(\d{1,5})\s/;
	my $rc = SarkSubs::request_syscmd ("/bin/kill -1 $1");
    }
		
		

    showInitial ($q,$response);
}


sub performDump ($)
{
    my $msg;
    my $mount = `/bin/mount`;
    unless ( $mount =~ /\/media\/usbdisk/ ) {
	if (`/bin/mount /media/usbdisk`) {
#		showInitial ($q, "<font color = red>@{[SarkSubs::tt_display('Sarkglobalsmsg7')]}</font color = red><font color = black>");
                showInitial ($q,"");
	}
    }
    my $date = time;
    unless ( -e  "/media/usbdisk/backups" ) {
        system ("/bin/mkdir /media/usbdisk/backups") == 0
		or die ("Couldn't create backups directory.\n") ;
    }
#    unless ( -e  "/tmp/backups" ) {
#        system ("/bin/mkdir /tmp/backups") == 0
#		or die ("Couldn't create /tmp/backups directory.\n") ;
#    }
#    system "rm -rf /tmp/backups/asterisk.sql";
#    system "/usr/bin/mysqldump asterisk > /tmp/backups/asterisk.sql";
    my $return = `/bin/tar -cf  /media/usbdisk/backups/asterisk.bak.$date.tar /home/e-smith/db/selintra  /usr/share/asterisk/sounds/usergreeting* /var/log/asterisk/cdr-csv/Master.csv /tmp/backups/asterisk.sql`;
    if ($return) {
    	$msg = "$return\n";
    }
#    $msg .= "Logs and identity dumped to asterisk.bak.$date on USB Flash";
#    showInitial ($q, $msg);
    showInitial ($q,"");
}

sub performRestore ($)
{
    unless ( `/bin/mount` =~ /\/media\/usbdisk/ ) {
	if (`/bin/mount /dev/sda1 /media/usbdisk`) {
#		showInitial ($q, "<font color = red>@{[SarkSubs::tt_display('Sarkglobalsmsg7')]}</font color = red><font color = black>");
                showInitial ($q,"");
	}
    }

   # SarkSubs::sark_header ($q, '', SarkSubs::tt_display('Sarkglobalsh214'), '');

    print $q->p ($q->b ("@{[SarkSubs::tt_display('Sarkglobalsh213')]}<br/><br/>"));

    my @Lines = (</media/usbdisk/backups/*>);

    print '<p align=center>';

    print $q->table ({border => 1, cellspacing => 1, cellpadding => 4});
#    print $q->Tr (esmith::cgi::genSmallCell ($q, $q->b ('Image Name'),"header"),
#				esmith::cgi::genSmallCell ($q, ('Size'),"header"),
#				esmith::cgi::genSmallCell ($q, ('Created'),"header"),
#                      	esmith::cgi::genSmallCell ($q, (''),"header"));

    foreach (@Lines)
        {
            my $header      = $_	|| '';
            my $size = (stat($_))[7];
		my $created = localtime((stat($_))[9]);

#            print $q->Tr (esmith::cgi::genSmallCell ($q, $header),
#				esmith::cgi::genSmallCell ($q, $size),
#				esmith::cgi::genSmallCell ($q, $created),
#                     	esmith::cgi::genSmallCell ($q,
#                    		$q->a ({href => $q->url (-absolute => 1)
#                        	. "?state=readyrestore&change=yes&line=$_"},
#					"<img src=\"/sark-common/edit.png\" border=0 >")));
    	  }


    print '</table>';
    print '</p>';
    print '<br/><br/>';


#
#  NO CANCEL BUTTON GRAPHIC YET
#
#    print $q->Tr (esmith::cgi::genButtonRow ($q, $q->submit (-name => 'cancel',
#    	                                                     -value => SarkSubs::tt_display('SarkbuttonCancel'),
#							     -style => "background-color:$buttoncolour")));
#    print image_button(-name=>'cancel',
#	    #			-id=>"mySave",
#                                -src=>'/sark-common/buttons/V1__0012_SAVE-ON.png',
#                                -style=>'border-style: none; margin: 4px 2px 4px 2px; width: 32px; height: 32px;',
#                                -onmouseover=>"javascript:this.src='/sark-common/buttons/V1__0022_SAVE-ROLLOVER.png';",
#                                -onmouseout=>"javascript:this.src='/sark-common/buttons/V1__0012_SAVE-ON.png';",
#                                -alt=>'Save and Continue',
#				-align=>'MIDDLE');
    SarkSubs::sark_footer($q);
}

sub readyRestore ($)
{
    my ($q) = @_;
    my $line     = $q->param ('line');

   # SarkSubs::sark_header ($q, '', SarkSubs::tt_display('Sarkglobalsh214'),'');

    print $q->p ("<font color=lightskyblue size=5> @{[SarkSubs::tt_display('Sarkglobalsmsg8')]}
			</font color=lightskyblue><font color = black><br/><br/><br/>");
    print $q->p ($q->a ({href => $q->url (-absolute => 1) . "?state=doRestore&line=$line"},
                  '<p align=center ><img src="/sark-common/warning.png" border=0 ></p><br/><br/><br/><br/>'));

#
#  NO CANCEL BUTTON GRAPHIC YET
#
#    print $q->Tr (esmith::cgi::genButtonRow ($q, $q->submit (-name => 'cancel',
#    	                                                     -value => SarkSubs::tt_display('SarkbuttonCancel'),
#							     -style => "background-color:$buttoncolour")));
#    print image_button(-name=>'cancel',
#	    #			-id=>"mySave",
#                                -src=>'/sark-common/buttons/V1__0012_SAVE-ON.png',
#                                -style=>'border-style: none; margin: 4px 2px 4px 2px; width: 32px; height: 32px;',
#                                -onmouseover=>"javascript:this.src='/sark-common/buttons/V1__0022_SAVE-ROLLOVER.png';",
#                                -onmouseout=>"javascript:this.src='/sark-common/buttons/V1__0012_SAVE-ON.png';",
#                                -alt=>'Save and Continue',
#
    print $q->hidden (-name => 'line',  -override => 1, -default => $line);

   SarkSubs::sark_footer($q);
}

sub doRestore ($)
{
    my ($q) = @_;
    my $line     = $q->param ('line');

    $line =~ /^(.+)$/;
    my $fqfn = $1;
    $line =~ /\/(asterisk\.bak\.\d+\.tar)$/;
    my $tarfile = $1;
#    print STDERR "TARFILE==>$tarfile\n";

    unless ( -e  "/tmp/backups" ) {
        system ("/bin/mkdir /tmp/backups") == 0
		or die ("Couldn't create /tmp/backups directory.\n") ;
    }

    system ("/bin/cp -fp  $fqfn /tmp/backups/") == 0
	or die ("Error occurred during selintra copy to /tmp.\n");
    chdir "/tmp/backups/";
    system ("/bin/tar xf $tarfile") == 0
    	or die ("Error occurred during untar.\n");
    system ("/bin/mv -f /tmp/backups/home/e-smith/db/selintra /home/e-smith/db/selintra-work") == 0
    	or die ("Error occurred moving selintra .\n");
    system ("/bin/mv -f /tmp/backups/usr/share/asterisk/sounds/usergreeting* /usr/share/asterisk/sounds/") == 0
    	or die ("Error occurred moving greetings .\n");

##    doCommit ();
    SarkSubs::sysCommit($q);

#    showInitial ($q, "<font color = red>@{[SarkSubs::tt_display('Sarkglobalsmsg9')]}</font color = red><font color = black>");
    showInitial ($q,"");
}

sub performStopStart ($$)  {
    my ($q, $stopstart) = @_;
    my $msg = ();
    my $action = ();

    if ($stopstart eq "START") {
#    	doCommit;
	$msg = ( StartAsterisk() );
    }

    if ($stopstart eq "HA ON") {
        StopAsterisk();
	my $dbh = SarkSubs::SQLiteConnect();
        SarkSubs::SQLiteDo($dbh, "UPDATE globals SET BINDADDR='ON' WHERE pkey='global'");
        SarkSubs::SQLiteDisconnect($dbh);
	SarkSubs::sysCommit($q);
        $msg = ( StartHa() );
    }

    if ($stopstart eq "STOP") {
	$msg = ( StopAsterisk() );
    }

    if ($stopstart eq "HAOFF") {
	my $dbh = SarkSubs::SQLiteConnect();
        SarkSubs::SQLiteDo($dbh, "UPDATE globals SET BINDADDR='OFF' WHERE pkey='global'");
        SarkSubs::SQLiteDisconnect($dbh);
	SarkSubs::sysCommit($q);
    	$msg = ( StopHa() );
    }

print STDERR "RUNNING SHOWINITIAL\n";
    showInitial ($q,$msg);
}
sub StopAsterisk ()
{
    my $ret = SarkSubs::sark_stop();
    return ("$ret");
}
sub StartAsterisk ()
{
    my $ret = SarkSubs::sark_start();
    return ("$ret");
}
sub StopHa ()
{
    my $rc;
    $rc = SarkSubs::request_syscmd ("/sbin/e-smith/config setprop heartbeat status disabled");
#    $rc = SarkSubs::request_syscmd ("/sbin/e-smith/config setprop shadow status disabled");
##    $rc = SarkSubs::request_syscmd ("/sbin/e-smith/config setprop syncher status disabled");
    $rc = SarkSubs::request_syscmd ("/sbin/e-smith/config setprop sark status enabled");
    $rc = SarkSubs::request_syscmd ("/etc/init.d/heartbeat stop");
#    $rc = SarkSubs::request_syscmd ("/etc/init.d/shadow stop");
    return ("HA Stopped!");
}
sub StartHa ()
{
    my $rc;
    my $dbh = SarkSubs::SQLiteConnect();
    my %global = SarkSubs::SQLiteGetHash($dbh, "SELECT * FROM globals WHERE pkey = 'global'");
    SarkSubs::SQLiteDisconnect($dbh);
    if ($global{HASECNODE}) {
       	if ($global{HACLUSTERIP}) {
               	$rc = SarkSubs::request_syscmd ("/sbin/e-smith/config setprop heartbeat status enabled");
#     		$rc = SarkSubs::request_syscmd ("/sbin/e-smith/config setprop shadow status enabled");
       		$rc = SarkSubs::request_syscmd ("/sbin/e-smith/config setprop sark status disabled");
#       		$rc = SarkSubs::request_syscmd ("/sbin/e-smith/config setprop syncher status enabled");
       		$rc = SarkSubs::request_syscmd ("/etc/init.d/heartbeat start");
#       		$rc = SarkSubs::request_syscmd ("/etc/init.d/shadow start");
                $rc = "HA Started!";
        }
        else {
                $rc = "HA Virtual IP not configured!";
        }
    }
    else {
    	$rc = "HA failover node not configured!";
    }

    return $rc;
}

sub printSysNotes () {
#
#   prints sysinfo Tab
#
    print '<div  class="notes">';
    print '<p class="last">';

	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime;
	$year += 1900;
	$mon += 1;
	my $systime = sprintf "%02d:%02d", $hour, $min;
    my $localIP			= SarkSubs::ret_localip();
    my $systemMode		= `/sbin/e-smith/config get SystemMode`;
    my $systemName		= `/bin/hostname`;
    my $domainName		= '';
#    my $externip		= ret_externip();
    my $snmask			= SarkSubs::ret_subnetmask();

    my $rlse       = `/bin/cat /opt/sark/.sail-release`;
    my @free = `/usr/bin/free`;
    my $uptime = `/usr/bin/uptime`;
    $uptime =~ /up\s(\d+\sdays)/;
    my $updays = $1;
    my $totmem, ;
    my $usedmem;
    my $freemem;
    my $sharedmem;
    my $buffers;
    my $cached;
    my $swaptot;
    my $swapused;
    my $swapfree;
    my @disk = `/bin/df -h`;
    $_ = $disk[2];
    /(\d{1,2}\%)/;
    my $diskusage = $1;
    my $systemMedia = 'disk';
    if (-e "/persistent") {
	$systemMedia = 'flash';
    }
    foreach (@free) {
	if (/Mem:\s+(\d{1,7})\s+(\d{1,7})\s+(\d{1,7})\s+(\d{1,7})\s+(\d{1,7})/) {
		$totmem = $1;
		$usedmem = $2;
		$freemem = $3;
		$sharedmem = $4;
		$buffers = $5;
	}
    }
    my $dbh = SarkSubs::SQLiteConnect();
    my $hasecnode = SarkSubs::SQLiteGetGlobal($dbh, 'HASECNODE');
    my $haclusterip = SarkSubs::SQLiteGetGlobal($dbh, 'HACLUSTERIP') || $localIP;
    my $extlen  = SarkSubs::SQLiteGetGlobal($dbh, 'EXTLEN');
    my $sno = rets();
    my $logopts = SarkSubs::SQLiteGetGlobal($dbh, 'LOGOPTS');
    my $runstate;
    my $harunstate;
    
    
    
    $ENV{SERVER_SOFTWARE} =~ '^(\w*)\b';
    my $server = $1;
    SarkSubs::SQLiteDisconnect($dbh);
    unless ( SarkSubs::check_pid() ) {
        $runstate = "STOPPED";
    }
    else {
        $runstate = "RUNNING";
    }
    if ( -e "/etc/init.d/heartbeat" ) {
    	unless ( SarkSubs::check_hapid() ) {
        	$harunstate = "STOPPED";
    	}
   	else {
        	$harunstate = "RUNNING";
    	}
    }
#
#   get the system type
#
    if ( -e "/etc/redhat-release" ) {
    	$_ = `/bin/cat /etc/redhat-release`;
    	/^(.*)beta.*$/;
    	my $stype = $1;
    }
#
#
#
    my $systate = "FULL";
    if ( -e "/opt/sark/.modeone") {
    		$systate = "DEMO";
    }

    print '<span style="color: #696969;" >';
    print '<span style="font-weight:bold; font-size:small; ">Sysinfo</span><br />';
    print "Serial #: <strong>$sno</strong><br/>";
    print "Release #: <strong>$rlse</strong><br/>";
#    print "Platform: <strong>$stype</strong><br/>";
    print "Web Server: <strong>$server</strong><br/>";
    $ENV{HTTP_USER_AGENT} =~ /(MSIE)/ ||
    	$ENV{HTTP_USER_AGENT} =~ /(Chrome)/ ||
        $ENV{HTTP_USER_AGENT} =~ /(Firefox)/;
    if ($1) {
    	print "Browser: <strong>$1</strong><br/>";
    }
    else {
        print "Browser: <strong>Unknown</strong><br/>";
    }
#    print '<br/>';
    print '<span style="font-weight:bold; font-size:small; ">Network</span><br />';
    print "hostname: <strong>$systemName</strong><br/>";
#    print "domain: <strong>$domainName</strong><br/>";
    print "LAN IP: <strong>$localIP</strong><br/>";
    if ( -e "/etc/init.d/heartbeat"  && $harunstate eq "RUNNING" ) {
        print "HAE IP: <strong>$haclusterip</strong><br/>";
    }
    print "Netmask: <strong>$snmask</strong><br/>";
#    print '<br/>';
    print '<span style="font-weight:bold; font-size:small; ">Hardware</span><br />';
    print "System Media: <strong>$systemMedia</strong><br/>";
    unless ( -e "/persistent" ) {
        print "Disk Usage: <strong>$diskusage</strong><br/>";
    }
    print "RAM Size: <strong>$totmem</strong><br/>";
    print "RAM Free: <strong>$freemem</strong><br/>";
#    print '<br/>';
    print '<span style="font-weight:bold; font-size:small; ">Status</span><br />';
    print "PBX State: <strong>$runstate</strong><br/>";
    if ( -e "/etc/init.d/heartbeat" ) {
        print "HAE State: <strong>$harunstate</strong><br/>";
    }
    print "SysTime: <strong>$systime</strong><br/>";
    if ($updays) {
    	print "System Uptime: <strong>$updays</strong><br/>";
    }
    print "</p>\n";
    print '</span>';
    print "</div>\n";

}

sub rets() {
  if (-e "/persistent") {
	if ( ! -e "/opt/sark/db/.sark-install-date" ) {
		`/bin/date > /opt/sark/db/.sark-install-date`;
	}
  }
  my $dir = "/opt/sark/passwd";
  my @list = <$dir/.*>;
  my $flg = 0;
  foreach (@list) {
        if (/\.(\d+)$/) {
                $flg = $1;
                last;
        }
  }
  unless ($flg) {
        $flg = int(rand(1000000));
        open( FH,">$dir/.$flg" )
                or die "Can't create: $!";
        close(FH);
  }
  return $flg;
}

sub Global_action ()
{
#
#   The button CGI is long winded so it's been moved to Subs
#
    print '<div  class="buttons">';

    SarkSubs::sark_button_save("save");
    SarkSubs::sark_button_commit("commit", $commit);
#    if (SarkSubs::SQLiteCheckUndo) {
#        SarkSubs::sark_button_undo();
#    }

    unless ( SarkSubs::check_pid ) {
                unless ( SarkSubs::check_hapid ) {
			SarkSubs::sark_button_start();
                }
    }
    else {
		SarkSubs::sark_button_stop();
    }

    if ( -e "/etc/init.d/heartbeat" ) {
    	unless ( SarkSubs::check_hapid ) {
        	SarkSubs::sark_button_ha_start();
    	}
        else {
        	SarkSubs::sark_button_ha_stop();
   	}
    }
#    my $starstate = SarkSubs::check_pid;
#    print STDERR "STARSTATE IS $starstate\n";
#    unless ( SarkSubs::check_pid ) {
#        SarkSubs::sark_lamp_stopped();
#    }
#    else {
#        SarkSubs::sark_lamp_running();
#    }
    print '</div>';

#    print '<div class="lamp">';

#    print '</div>';
}
