#!/usr/bin/perl
#
#----------------------------------------------------------------------
# heading     : Telephony
# description : Extensions
# navigation  : 4000 4810
#
# Copyright (c) selintra.com         2005
#
#----------------------------------------------------------------------
#----------------------------------------------------------------------
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
#
# Technical support for this program is available from Selintra Ltd
# Please visit our web site www.selintra.com/ for details.
#----------------------------------------------------------------------

my @Distro = `/bin/cat /etc/*-release`;
foreach (@Distro) {
	if ( /Ubuntu/ ) {
		use lib '/usr/lib/perl5/site_perl/5.8.8/';
		last;
	}
}

use lib '/opt/sark/perl/modules';


use strict;
use CGI ':all';
use CGI::Carp qw(fatalsToBrowser);
use Asterisk::AMI::Common;
use sark::SarkSubs;
#use sark::sarkvars qw ($editicon $yesicon $noicon $warnicon $infoicon $greenicon $activeicon $monicon $trashicon);


sub showInitial ($$);

sub commit ($);
sub createIPphone ($);
sub performCreateIPphone ($);
sub adoptIPphone ($);
sub linesIn ($);
sub performLinesIn ($);
sub performDelete ($);
sub peerState($);

BEGIN
{
    # Clear PATH and related environment variables so that calls to
    # external programs do not cause results to be tainted. See
    # "perlsec" manual page for details.

    $ENV {'PATH'} = '';
    $ENV {'SHELL'} = '/bin/bash';
    delete $ENV {'ENV'};

}

$< = $>;
$( = $);

$CGI::POST_MAX=1024 * 100;  # max 100K posts
$CGI::DISABLE_UPLOADS = 1;  # no uploads


my $editicon = "<img src=\"/sark-common/edit.png\" border=0 title = \"@{[SarkSubs::tt_display('Sarkedittext')]}\">";
my $yesicon = "<img src=\"/sark-common/connected.png\" border=0 title = \"@{[SarkSubs::tt_display('Sarktinfotext')]}\">";
my $noicon = "<img src=\"/sark-common/actions/no.png\" border=0 title = \"@{[SarkSubs::tt_display('Sarkoffline')]}\" >";
my $warnicon = "<img src=\"/sark-common/warning.png\" border=0 title = \"@{[SarkSubs::tt_display('Sarkwarntext')]}\" >";
my $infoicon = "<img src=\"/sark-common/info.png\" border=0 >";
my $greenicon = "<img src=\"/sark-common/actions/apply.png\" border=0 title = \"@{[SarkSubs::tt_display('Sarkremotetext')]}\" >";
my $activeicon = "<img src=\"/sark-common/actions/apply.png\" border=0 title = \"@{[SarkSubs::tt_display('Sarkactivetext')]}\" >";
my $monicon = "<img src=\"/sark-common/actions/apply.png\" border=0 title = \"@{[SarkSubs::tt_display('Sarkmontext')]}\" >";
my $trashicon = "<img src=\"/sark-common/edittrash.png\" border=0 onclick=\"return confirmOK('Delete this Object - Confirm?')\" title = \"@{[SarkSubs::tt_display('Sarkdeletetext')]}\" >";

my $notifyicon = "<img src=\"/sark-common/actions/reload.png\" border=0 onclick=\"return confirmOK('Reload this endpoint? - Confirm?')\" title = \"@{[SarkSubs::tt_display('Sarknotifytext')]}\" >";


my %hashScrnVars;
my @Verrors;
my $gonogo = 0;
my $cfwd;
my %hashVarDefaults = (
        "passwd" 	=> undef,
        "cluster"  	=> undef,
        "device"	=> undef,
        "desc"		=> undef,
	"dialstring"    => undef,
        "technology"   	=> undef,
        "macaddr"       => undef,
	"devicerec" 	=> "default",
        "username"      => undef,
        "desc" 		=> undef,
        "vmailfwd"   	=> undef,
        "dvrvmail"	=> undef,
        "callerid"	=> undef,
        "extalert"     	=> undef,
        "location" 	=> undef,
        "pkey" 		=> undef,
        "provision"  	=> undef,
        "sipiaxfriend" 	=> undef,
        );
my %hashVarErrors;
my %hashVarOutBuf = (
	"callerid" 	=> undef,
	"cluster" 	=> undef,
	"desc" 		=> undef,
	"device" 	=> undef,
	"devicerec"	=> undef,
	"dialstring"    => undef,
	"dvrvmail" 	=> undef,
        "extalert" 	=> undef,
	"location" 	=> undef,
	"macaddr"	=> undef,
	"passwd" 	=> undef,
        "pkey" 		=> undef,
	"provision" 	=> undef,
	"sipiaxfriend" 	=> undef,
	"stealtime" 	=> undef,
	"stolen" 	=> undef,
	"technology" 	=> undef,
	"vmailfwd" 	=> undef
        );
my %hashVarUpdBuf = ();
my %hashVarRegexs = (
    	"AADEFAULT" 	=> qr/(.*)/,
	"callerid" 	=> qr/^(\w{3,18})$/,
        "desc"    	=> qr/^(\w{2,20})$/,
	"dialstring"    => qr/^([\w\/-]+)$/,
        "dvrvmail" 	=> qr/^(\w{3,4})$/,
        "macaddr"  	=> qr/^([0-9a-fA-F:]{12,18})$/,
        "passwd"  	=> qr/^(\w{2,20})$/,
        "pkey"  	=> qr/^(\d{3,4})$/,
        "provision" 	=> qr/^(.*)$/ms,
        "sipiaxfriend" 	=> qr/^(.*)$/ms,
        "vmailfwd" 	=> qr/(^\w[-\._\w]*\w@\w[-._\w]*\w\.\w{2,3}$)/,
        );

my %hashColorDefaults = %hashVarDefaults;
while ( my $tk = each %hashColorDefaults ) {
        $hashColorDefaults{$tk} = 'black';
}
my %hashcallstate = (
    	"-1" 	=> "ND",
	"0" 	=> "Idle",
        "1"    	=> "Call",
        "2" 	=> "Busy",
        "4"  	=> "NR",
        "8"  	=> "Ring",
        "16"  	=> "Hold",
        );
my $buttoncolour = 'lightskyblue';
#my $maxundonum = '5';
my $port;
my $peerstate = $noicon;
my $hostip;
my $device;
my @IPcandidate;
my $pages = 1 ;
my $page = 1;
my $epages = 1;
my $epage = 1;

my $q = new CGI;
my %params = $q->Vars;

#
#  Retrieve the page number
#
for my $key ( keys %params ) {
       if ( $key =~ /^page(\d{1,}).x$/ ) {
       	  $page=$1;
          showInitial ($q, '');
          exit(0);
       }
       if ( $key =~ /^epage(\d{1,}).x$/ ) {
       	  $epage=$1;
          linesIn ($q);
          exit(0);
       }
}

if ($q->param ('new.x') )  {
	createIPphone ($q);
}

elsif ($q->param ('adopt.x') ) {
	adoptIPphone ($q);
}

elsif ($q->param ('phase1.x') ) {
	performCreateIPphone ($q);
}

elsif ($q->param ('initcommit.x') ) {
	commit ($q);
}

elsif ($q->param ('newcommit.x') ) {
	performCreateIPphone ($q);
}

elsif ($q->param ('commit.x') ) {
    performLinesIn ($q);
}

elsif ($q->param ('cfwd.x') ) {
    $cfwd = "YES";
    linesIn ($q);
}
elsif ($q->param ('srchkey') ) {
    linesIn ($q);
}

elsif ($q->param ('state') eq "linesIn")
{
    linesIn ($q);
}

elsif ($q->param ('state') eq "orphansIn")
{
	createIPphone ($q);
}


elsif ($q->param ('upsave.x') ) {
    performLinesIn ($q);
}

elsif ($q->param ('cancel.x')) {
    showInitial ($q, '');
}

elsif ($q->param ('delete.x') ) {
    performDelete ($q);
}
elsif (! grep (/^state$/, $q->param) || $q->param ('resolve.x') ) {
    showInitial ($q, '');
}
else
{
    die "STATE ERROR!! ";
}




exit (0);

#------------------------------------------------------------
# subroutine to display initial form
#------------------------------------------------------------

sub showInitial ($$)
{
    my $exists = ();
    my ($q, $msg) = @_;
    my $pbx = 'SARK';
    my $noasterisk;
    my $numexts = 0;
    my $numup = 0;
    my $numdown = 0;
    my $rowcolor = 0x01;
    my $pkey;

    my $astman;
# astman doesn't always work first time - so we retry    
    for (my $i=1; $i<3; $i++) {
		$astman = Asterisk::AMI::Common->new(
        PeerAddr        =>      '127.0.0.1',
    	PeerPort        =>      '5038',
        Username        =>      'sark',
        Secret          =>      'mysark'
        );
        if ($astman) {
			last;
		}
	}
    unless ($astman) {
       	$noasterisk = "N/A";
    }
    my $dbh = SarkSubs::SQLiteConnect();
    my $commit = SarkSubs::SQLiteGet($dbh, "SELECT MYCOMMIT FROM globals where pkey = 'global'");
    my @IPphone = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM IPphone" ));
    my $numphones = @IPphone;
    my $proxyonoff = SarkSubs::SQLiteGet($dbh, "SELECT PROXY FROM globals where pkey = 'global'");
    SarkSubs::sark_header ($q);
    my $remainder = $numphones % 12;
    $pages = int($numphones / 12);
    if ($remainder) {
    	$pages++;
    }
    my $linelow = (($page-1) * 12);
    my $linehigh = $linelow + 12;

    my $printline = SarkSubs::tt_display('Sarksystemh11').($msg || "Extensions ") ;
    unless ($astman ) {
       $printline = SarkSubs::tt_display('Sarksystemh11')."Cannot retrieve live values from Asterisk";
    }
    print h1( $printline );
    print '<hr class="hr" />';

    print '<div class="buttons">';
    unless (sark_mode($numphones)) {
        SarkSubs::sark_button_new();
    }
    SarkSubs::sark_button_commit("initcommit",$commit);
# no adopt for PIKA
    unless ( -e "/persistent" ) {
    	SarkSubs::sark_button_adopt();
    }

    print '</div>';


    print '<div>';

    print "<div class=etabs>";

    unless (@IPphone) {
        print '<span style="color:red;font-weight:bold">'.SarkSubs::tt_display('Sarkinternalmsg1').'</span>';
        print '</div></div>';
        SarkSubs::sark_footer($q);
        SarkSubs::SQLiteDisconnect($dbh);
        exit(0);
    }

    #show table
    else {
        print $q->table ({border => 1, cellspacing => 1, cellpadding => 4});

        print $q->Tr (
        	      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('ext')),
                      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('cluster')),
                      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('uname')),
                      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('device')),
#                      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('port')),
                      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('macaddr')),
                      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('ipaddr')),
                      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('latency')),
#                      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('devicerec')),
                      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('callstate')),
                      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('headlocation')),
                      $q->th({class=>"srk-border"}, ''),
                      $q->th({class=>"srk-border"}, ''),
                      $q->th({class=>"srk-border"}, ''),
                      $q->th({class=>"srk-border"}, '')
                      );

		my $sip_peers;
		unless ($noasterisk) {  
			$sip_peers = $astman->sip_peers();
		}
        my $latency = "N/A" ;
        my $formalMac ;
        my $hostip = "N/A";;
        my $port = "N/A" ;
        my $nstate = "NR";
        my $callstate = "NR" ;
        my @Arp = `/bin/cat /proc/net/arp`;
        foreach (@IPphone) {
            $pkey = $_;
 #            my %IPphone = SarkSubs::SQLiteGetHash($dbh, "SELECT * FROM IPphone WHERE pkey = '$pkey'");
 # just the fields we need saves 20- 80 ms
 #            my %IPphone = SarkSubs::SQLiteGetHashKey($dbh, "SELECT location, macaddr, technology, desc, cluster, device, devicerec  FROM IPphone WHERE pkey = ?",$pkey);
 # regressed this because it wasn't closing the query properly
 #
			my %IPphone = SarkSubs::SQLiteGetHash($dbh, "SELECT * FROM IPphone WHERE pkey = '$pkey'");

            $callstate = "ND" ;
            $numexts++;

            if ($IPphone{'location'} eq 'remote') {
            	$IPphone{'macaddr'} = 'N/A';
            }
            unless ($IPphone{'macaddr'}) {
                $IPphone{'macaddr'} = 'N/A';
            }
         if ($IPphone{'technology'} eq 'SIP') {
			if (defined $sip_peers ) {
                 $latency = $sip_peers->{$pkey}{Status};
                 $hostip = $sip_peers->{$pkey}{IPaddress};
                 $port = $sip_peers->{$pkey}{IPport};
            }
            if ($port eq "0") {
                $port = "";
            }
            if ($latency eq 'UNKNOWN') { $latency = 'N/A'; }
            $formalMac = $IPphone{'macaddr'} || 'N/A';
            if ($latency eq 'UNREACHABLE') { $latency = 'U/R'; }
            $formalMac = $IPphone{'macaddr'} || 'N/A';

            
            if ($astman) { $nstate = $astman->exten_state($pkey,'extensions');}
            $callstate = $hashcallstate{$nstate};
            
            $peerstate = $noicon;
            if ($latency =~ /OK/ ) {
            	$latency =~ /^OK \((.*)\)$/;
                $latency = $1;
            	$numup++;
            	if ($IPphone{'location'} eq "remote" || $proxyonoff eq "NO" || -e "/persistent" ) {
                	$peerstate = $greenicon;
				}
				else {
					$peerstate = $yesicon;
				}
            }
            else {
            	# try to find the phone from ARP
                unless ( $formalMac eq "N/A" ) {
                	$formalMac =~ s/(..)(..)(..)(..)(..)(..)/\U$1:$2:$3:$4:$5:$6/;
                	foreach (@Arp) {
                		if ( /^(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}).* (([0-9A-F]{2}:){5}[0-9A-F]{2}) / ) {
							if ( $formalMac eq $2 ) {
                        			$peerstate = $warnicon;
                        			$hostip = $1;
                            }
            			}
                	}
            	}
            }
    	 }
     	 else	{
			$peerstate = $greenicon;
			$numup++;
			if ($IPphone{'technology'} eq 'IAX2') {
				$hostip = 'IAX2';
			}

			elsif ($IPphone{'technology'} eq 'Analogue') {
				$hostip = 'Analogue';
                $port = 'N/A';
                $IPphone{technology} = "TDM";
			}
			elsif ($IPphone{'technology'} eq 'Custom') {
                $hostip = 'Custom';
                $port = 'N/A';
                $IPphone{technology} = "Custom";
			}
			else {
				$hostip = 'N/A - Unknown';
			}
		}
	if ( length $IPphone{cluster} > 9 ) {
		$IPphone{cluster} = substr($IPphone{cluster} , 0, 8);
		$IPphone{cluster} .= '.';
    }
#    $IPphone{device} =~ /(\w+)*/;
#    $IPphone{device} = $1;
    if ( length $IPphone{device} > 9 ) {
		$IPphone{device} = substr($IPphone{device} , 0, 8);
		$IPphone{device} .= '.';
    }
    
    if ( length $IPphone{desc} > 8 ) {
		$IPphone{desc} = substr($IPphone{desc} , 0, 7);
		$IPphone{desc} .= '.';
    }
    
    if ($IPphone{technology} eq 'SIP') {
		if ($peerstate eq $yesicon) {
        	    print $q->Tr ({-class => "d".$rowcolor},
                    	  $q->td({class=>"srk-border"}, "$IPphone{technology}/$pkey"),
                          $q->td({class=>"srk-border"}, $IPphone{cluster}),
                          $q->td({class=>"srk-border"}, $IPphone{desc}),
                          $q->td({class=>"srk-border"}, $IPphone{device}),
#                          $q->td({class=>"srk-border"}, $port),
                          $q->td({class=>"srk-border"}, $IPphone{macaddr}),
                          $q->td({class=>"srk-border"}, $hostip),
                          $q->td({class=>"srk-border"}, $latency),
#                          $q->td({class=>"srk-border"}, $IPphone{devicerec}),
                          $q->td({class=>"srk-border"}, $callstate),
                          $q->td({class=>"srk-border"}, $IPphone{location}),
                          $q->td({class=>"srk-border"},
                          	$q->a ({href => "/DPRX$hostip/"},
                                $peerstate)),
                     	  $q->td({class=>"srk-border"},
                    	  	$q->a ({href => $q->url (-absolute => 1)
                        	. "?state=linesIn&change=yes&pkey=$pkey"},
							$editicon)),
						  $q->td({class=>"srk-border"},
                    	  	$q->a ({href => $q->url (-absolute => 1)
                        	. "?state=linesIn&notify=yes&pkey=$pkey"},
							$notifyicon)), 
                          $q->td({class=>"srk-border"},
                        	$q->a ({href => $q->url (-absolute => 1)
                        	. "?state=linesIn&delete=yes&pkey=$pkey"},
							$trashicon)));
		}
		elsif ($peerstate eq $warnicon) {
        	    print $q->Tr ({-class => "d".$rowcolor},
                    	  $q->td({class=>"srk-border"}, "$IPphone{technology}/$pkey"),
                          $q->td({class=>"srk-border"}, $IPphone{cluster}),
                          $q->td({class=>"srk-border"}, $IPphone{desc}),
                          $q->td({class=>"srk-border"}, $IPphone{device}),
#                          $q->td({class=>"srk-border"}, $port),
                          $q->td({class=>"srk-border"}, $IPphone{macaddr}),
                          $q->td({class=>"srk-border"}, $hostip),
                          $q->td({class=>"srk-border"}, $latency),
#                          $q->td({class=>"srk-border"}, $IPphone{devicerec}),
                          $q->td({class=>"srk-border"}, $callstate),
                          $q->td({class=>"srk-border"}, $IPphone{location}),
                          $q->td({class=>"srk-border"},
                          	$q->a ({href => "/DPRX$hostip/"},
                                $peerstate)),
                     	  $q->td({class=>"srk-border"},
                    	  	$q->a ({href => $q->url (-absolute => 1)
                        	. "?state=linesIn&change=yes&pkey=$pkey"},
							$editicon)),
#						  $q->td({class=>"srk-border"},
#                    	  	$q->a ({href => $q->url (-absolute => 1)
#                        	. "?state=linesIn&notify=yes&pkey=$pkey"},
#							$notifyicon)), 
						  $q->td({class=>"srk-border"}, $noicon),
                          $q->td({class=>"srk-border"},
                        	$q->a ({href => $q->url (-absolute => 1)
                        	. "?state=linesIn&delete=yes&pkey=$pkey"},
							$trashicon)));
		}
		elsif ($peerstate eq $greenicon) {
                    print $q->Tr ({-class => "d".$rowcolor},
                    	  $q->td({class=>"srk-border"}, "$IPphone{technology}/$pkey"),
                          $q->td({class=>"srk-border"}, $IPphone{cluster}),
                          $q->td({class=>"srk-border"}, $IPphone{desc}),
                          $q->td({class=>"srk-border"}, $IPphone{device}),
#                          $q->td({class=>"srk-border"}, $port),
                          $q->td({class=>"srk-border"}, $IPphone{macaddr}),
                          $q->td({class=>"srk-border"}, $hostip),
                          $q->td({class=>"srk-border"}, $latency),
#                          $q->td({class=>"srk-border"}, $IPphone{devicerec}),
                          $q->td({class=>"srk-border"}, $callstate),
                          $q->td({class=>"srk-border"}, $IPphone{location}),
                     	  $q->td({class=>"srk-border"}, $peerstate),
                     	  $q->td({class=>"srk-border"},
                    		$q->a ({href => $q->url (-absolute => 1)
                        	. "?state=linesIn&change=yes&pkey=$pkey"},
							$editicon)),
						  $q->td({class=>"srk-border"},
                   	  	    $q->a ({href => $q->url (-absolute => 1)
                        	. "?state=linesIn&notify=yes&pkey=$pkey"},
							$notifyicon)), 
                          $q->td({class=>"srk-border"},
                        	$q->a ({href => $q->url (-absolute => 1)
                        	. "?state=linesIn&delete=yes&pkey=$pkey"},
							$trashicon)));
		}		
		else {
                    print $q->Tr ({-class => "d".$rowcolor},
                    	  $q->td({class=>"srk-border"}, "$IPphone{technology}/$pkey"),
                          $q->td({class=>"srk-border"}, $IPphone{cluster}),
                          $q->td({class=>"srk-border"}, $IPphone{desc}),
                          $q->td({class=>"srk-border"}, $IPphone{device}),
#                          $q->td({class=>"srk-border"}, $port),
                          $q->td({class=>"srk-border"}, $IPphone{macaddr}),
                          $q->td({class=>"srk-border"}, $hostip),
                          $q->td({class=>"srk-border"}, $latency),
#                          $q->td({class=>"srk-border"}, $IPphone{devicerec}),
                          $q->td({class=>"srk-border"}, $callstate),
                          $q->td({class=>"srk-border"}, $IPphone{location}),
                     	  $q->td({class=>"srk-border"}, $peerstate),
                     	  $q->td({class=>"srk-border"},
                    		$q->a ({href => $q->url (-absolute => 1)
                        	. "?state=linesIn&change=yes&pkey=$pkey"},
							$editicon)),
#						  $q->td({class=>"srk-border"}, $noicon),
#						  $q->td({class=>"srk-border"},
#                   	  	$q->a ({href => $q->url (-absolute => 1)
#                        	. "?state=linesIn&notify=yes&pkey=$pkey"},
#							$notifyicon)), 
						  $q->td({class=>"srk-border"}, $noicon),
                          $q->td({class=>"srk-border"},
                        	$q->a ({href => $q->url (-absolute => 1)
                        	. "?state=linesIn&delete=yes&pkey=$pkey"},
							$trashicon)));
		}
	}
	else {
			print $q->Tr ({-class => "d".$rowcolor},
                    	  $q->td({class=>"srk-border"}, "$IPphone{technology}/$pkey"),
                          $q->td({class=>"srk-border"}, $IPphone{cluster}),
                          $q->td({class=>"srk-border"}, $IPphone{desc}),
                          $q->td({class=>"srk-border"}, $IPphone{device}),
#                          $q->td({class=>"srk-border"}, $port),
                          $q->td({class=>"srk-border"}, $IPphone{macaddr}),
                          $q->td({class=>"srk-border"}, $hostip),
                          $q->td({class=>"srk-border"}, $latency),
#                          $q->td({class=>"srk-border"}, $IPphone{devicerec}),
                          $q->td({class=>"srk-border"}, $callstate),
                          $q->td({class=>"srk-border"}, $IPphone{location}),
                     	  $q->td({class=>"srk-border"}, $peerstate),
                     	  $q->td({class=>"srk-border"},
                    		$q->a ({href => $q->url (-absolute => 1)
                        	. "?state=linesIn&change=yes&pkey=$pkey"},
							$editicon)),
						  $q->td({class=>"srk-border"}, $noicon),
                          $q->td({class=>"srk-border"},
                        	$q->a ({href => $q->url (-absolute => 1)
                        	. "?state=linesIn&delete=yes&pkey=$pkey"},
							$trashicon)));		
	}
    $rowcolor = $rowcolor ^ 1;
#           $i++;
	}

        print '</table>';

#        print '</p>';
    }
    print '</div>';
    print '</div>';

    SarkSubs::sark_footer($q);
    SarkSubs::SQLiteDisconnect($dbh);
}

#------------------------------------------------------------
# phones
#------------------------------------------------------------

sub commit ($)
{
    SarkSubs::sysCommit($q);
    showInitial ($q, "<font color = red>@{[SarkSubs::tt_display('Sarksystemmsg1')]}</font color = red><font color = black>");
}

sub adoptIPphone ($)
{
    my $exists = ();
    my ($q) = shift;

    my $srchmask = SarkSubs::ret_localip();
    my $device = ();
    my $hostip = ();
    my $macaddr = ();
    my %IPphone = ();

    my $dbh = SarkSubs::SQLiteConnect();
    my @IPphonekeys = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM IPphone where macaddr!=''" ));

    $srchmask =~ s/\.\d{1,3}$/\.\*/;

    foreach (@IPphonekeys) {
    	my $temp = SarkSubs::SQLiteGet($dbh, "SELECT macaddr FROM IPphone where pkey = '$_'");
	$temp =~ s/(..)(..)(..)(..)(..)(..)/\U$1:$2:$3:$4:$5:$6/;
	$IPphone{$temp} = $_;
    }

    my @IPcandidate = ();
    my @Orphans = `/bin/cat /proc/net/arp`;

    foreach (@Orphans) {
#   Xlate for PIKA linux
	tr/a-f/A-F/;
    	if ( /([0-9A-F]{2}:[0-9A-F]{2}:[0-9A-F]{2}):[0-9A-F]{2}:[0-9A-F]{2}:[0-9A-F]{2}/ ) {
               	if (SarkSubs::SQLiteGet($dbh, "SELECT pkey FROM mfgmac where pkey = '$1'") ) {
               		push (@IPcandidate, $_);
                }
        }
    }

    my $IPcandidate = @IPcandidate || 'no phones';

    my $proxy = SarkSubs::SQLiteGet($dbh, "SELECT PROXY FROM globals where pkey = 'global'");

    SarkSubs::sark_header ($q);
    print h1( SarkSubs::tt_display('Sarksystemh11')."Orphans" );
    print '<hr class="hr" />';
    print "<div>";
    if ($IPcandidate eq 'no phones')
    {
        print $q->p ("<font color = red><b>@{[SarkSubs::tt_display('Sarkinternalmsg2')]}<font color = black></b>");
    }


    #show table
    else
    {
        print "<div class=aetabs>";
        print '<p align=center>';

        print $q->table ({border => 1, cellspacing => 1, cellpadding => 4});

        print $q->Tr (
                      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('device')),
		      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('macaddr')),
                      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('ipaddr')),
                      $q->th({class=>"srk-border"}, ''),
                      $q->th({class=>"srk-border"}, '')
                      );

        my $rowcolor = 0x01;

        foreach (@IPcandidate) {
            $peerstate = $warnicon;
            if ( /^(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}).* (([0-9A-F]{2}:){5}[0-9A-F]{2}) / ) {
               $hostip = $1;
               $macaddr = $2;
               $macaddr =~ /([0-9A-F]{2}:[0-9A-F]{2}:[0-9A-F]{2}):[0-9A-F]{2}:[0-9A-F]{2}/;
	       $device = SarkSubs::SQLiteGet($dbh, "SELECT name FROM mfgmac where pkey = '$1'");
	    }

            unless ( $IPphone{$macaddr} ) {
	           	print $q->Tr ({-class => "d".$rowcolor},
                        $q->td({class=>"srk-border"}, $device),
                        $q->td({class=>"srk-border"}, $macaddr),
                        $q->td({class=>"srk-border"}, $hostip),
                     	$q->td({class=>"srk-border"},
				$q->a ({href => "/DPRX$hostip/"},
				$warnicon)),
                     	$q->td({class=>"srk-border"},
                    		$q->a ({-class=> "d3", href => $q->url (-absolute => 1)
                        	. "?state=orphansIn&adopt=yes&macaddr=$macaddr&device=$device", title => SarkSubs::tt_display('Sarkadopttext') },
#				"<img src=\"/sark-common/edit.png\" border=0 >")));
                       	 	"Adopt")));

#		  }
                  $rowcolor = $rowcolor ^ 1;
	    }
        }
        print '</table></p>';
    }
   print "</div>";
   SarkSubs::sark_footer($q);
   SarkSubs::SQLiteDisconnect($dbh);

}
sub createIPphone ($)
{
    my ($q) = shift;
    my @Open;
    my @Closed;
    my $macaddr;
    my $device;

    if ( $q->param ('state') eq "orphansIn" ) {
        $device   = $q->param ('device');
    	$macaddr  = $q->param ('macaddr');
    	$macaddr =~ s/://g;
    }
    my $dbh = SarkSubs::SQLiteConnect();
    my $pkey = SarkSubs::SQLiteGet($dbh, "SELECT SIPIAXSTART FROM globals where pkey = 'global'");
    my $commit = SarkSubs::SQLiteGet($dbh, "SELECT MYCOMMIT FROM globals where pkey = 'global'");
    my $cosstart = SarkSubs::SQLiteGet($dbh, "SELECT COSSTART FROM globals where pkey = 'global'");
    my @IPphones = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM IPphone" ));
    my @Clusters = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM Cluster"));
    my @Cos = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM COS"));
    my @Devices = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM Device WHERE technology='SIP' OR technology='IAX2' OR technology='Custom'"));
    my $counter = 0;
    if ( $device ) {
    	while ($counter <= $#Devices) {
        	unless ( @Devices[$counter] =~ /$device/ ) {
                	splice ( @Devices,$counter,1 );
                }
                else {
                        $counter++;
                }
        }
    }
    while (SarkSubs::SQLiteGet($dbh, "SELECT pkey FROM IPphone where pkey = '$pkey'"))  {
	$pkey++;
    }

    foreach (@Cos) {
    	push (@Open, $_)
        	if (SarkSubs::SQLiteGet($dbh, "SELECT defaultopen FROM COS where pkey = '$_'") eq "YES");
        push (@Closed, $_)
        	if (SarkSubs::SQLiteGet($dbh, "SELECT defaultclosed FROM COS where pkey = '$_'") eq "YES" );
    }

    SarkSubs::SQLiteDisconnect($dbh);

    SarkSubs::sark_header ($q);
    print h1( SarkSubs::tt_display('Sarksystemh11').'Add an Extension' );
    print '<hr class="hr" />';
    if (@Verrors) {
	SarkSubs::sark_error(\@Verrors);
    }
    print '<div class="buttons">';
    SarkSubs::sark_button_cancel();
    SarkSubs::sark_button_save("phase1");
    SarkSubs::sark_button_commit("newcommit",$commit);
    print "</div>";

    print "<div>";
    print "<div class=\"notes\">
        <h4>Extension Details</h4>
        <p class=\"last\">Fill out the details for the new extension...
        	<strong>N.B.</strong>
                If you wish to use SARK's autoprovisioning features then you must provide the MAC
                address of the phone and choose the correct phone type from the <strong>Device</strong> drop down.
                If you are unsure of the phone type or you do not wish to use provisioning
                then you can simply choose General SIP or General IAX.
        	</p>
                </div>";



    my $pbx = 'SARK';


    print SarkSubs::aLabelFor('rule');
    print $q->textfield (
                    -name     => 'pkey',
                    -override => 1,
                    -default  => $pkey,
                    -size     => 4);
    print SarkSubs::aLabelFor('device');
    print $q->popup_menu (
                     -name    => 'device',
                     -values  => [@Devices, 'General SIP'],
                     -default => $device);
    print SarkSubs::aLabelFor('devicerec');
    print $q->popup_menu (
                     -name    => 'devicerec',
                     -values  => [SarkSubs::get_rec_list,'Inbound','Outbound','Both'],
                     -default => "default");
    print SarkSubs::aLabelFor('vmailfwd');
    print $q->textfield (
                     -name     => 'vmailfwd',
                     -override => 1,
                     -style    => "color: $hashColorDefaults{vmailfwd}",
                     -default  => $hashScrnVars{vmailfwd},
                     -size     => 20);

    print SarkSubs::aLabelFor('macaddr');
    print $q->textfield (
                     -name     => 'macaddr',
                     -override => 1,
                     -style    => "color: $hashColorDefaults{macaddr}",
                     -default  => $macaddr,
                     -size     => 12);
    print SarkSubs::aLabelFor('calleridname');
    print $q->textfield (
                     -name     => 'desc',
                     -override => 1,
                     -style    => "color: $hashColorDefaults{desc}",
#                     -default  => "Ext$pkey",
        	     -size     => 20);

    print SarkSubs::aLabelFor('location');
    print $q->popup_menu (
                     -name    => 'location',
                     -values  => ['local', 'remote'],
                     -default => 'local');
    print SarkSubs::aLabelFor('cluster');
    print $q->popup_menu (
                     -name    => 'cluster',
                     -values  => ['default',@Clusters],
                     -default => 'default');
    print SarkSubs::aLabelFor('extalert');
    print $q->textfield (
                    -name     => 'extalert',
                    -override => 1,
                    -style    => "color: $hashColorDefaults{extalert}",
                    -default  => $hashScrnVars{extalert},
                    -size     => 30);
#    print SarkSubs::aLabelFor('callerid');
#    print $q->textfield (
#                    -name     => 'callerid',
#                    -override => 1,
#                    -style    => "color: $hashColorDefaults{callerid}",
#                    -default  => $pkey,
#                    -size     => 15);
    print SarkSubs::aLabelFor('cdialstring');
    print $q->textfield (
                    -name     => 'dialstring',
                    -override => 1,
                    -default  => $hashScrnVars{dialstring},
                    -size     => 30);
    print "</div>";
#    print $q->hidden (
#                    -name     => 'dvrvmail',
#                    -override => 1,
#                    -default  => $pkey);
    SarkSubs::sark_footer($q);
}


sub performCreateIPphone ($)
{
    my ($q) = shift;
    %hashScrnVars = $q->Vars;


    my $dbh 			= SarkSubs::SQLiteConnect();
    my $sipiaxfriend 		= SarkSubs::SQLiteGet($dbh, "SELECT sipiaxfriend FROM Device where pkey = '$hashScrnVars{device}'");
    $hashScrnVars{technology}	= SarkSubs::SQLiteGet($dbh, "SELECT technology FROM Device where pkey = '$hashScrnVars{device}'");
    my $provision 		= SarkSubs::SQLiteGet($dbh, "SELECT provision FROM Device where pkey = '$hashScrnVars{device}'");
    my $extlen 			= SarkSubs::SQLiteGet($dbh, "SELECT EXTLEN FROM globals where pkey = 'global'") || '4';
    my $acl 			= SarkSubs::SQLiteGet($dbh, "SELECT ACL FROM globals where pkey = 'global'") || 'NO';
    my $tuple_exists 		= SarkSubs::SQLiteGet($dbh, "SELECT pkey FROM IPphone where pkey = '$hashScrnVars{pkey}'");
    my @Cos 			= SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM COS");
    $hashScrnVars{passwd} 	= SarkSubs::sark_password;
    SarkSubs::SQLiteDisconnect($dbh);

    my $ipaddr   	= SarkSubs::ret_localip();
#    my $cfgfilename     = '';
#    my @files = ();
#    my $undonum = db_get_prop(\%selintra, 'global', 'UNDONUM')   || '0';



    unless ($hashScrnVars{pkey} =~ /^\d{$extlen}$/) {
        $gonogo = "<font color = red> @{[SarkSubs::tt_display('Sarkinternalmsg3')]}</font color = red><font color = black>";
    	showInitial ($q, $gonogo);
	return;
    }

    if  ($tuple_exists) {
        showInitial ($q, "<font color = red> DATABASE KEY ALREADY EXISTS!</font color = red><font color = black>");
    	return;
    }

    unless ($hashScrnVars{desc}) {
	$hashScrnVars{desc} = "Ext$hashScrnVars{pkey}";
    }

    $hashScrnVars{callerid} = $hashScrnVars{pkey};

    if ($sipiaxfriend =~ /callerid=/) {
     		$sipiaxfriend =~ s/callerid=/callerid=\"$hashScrnVars{desc}\" <$hashScrnVars{pkey}>/;
    }
    else {
		$sipiaxfriend .= "\ncallerid=\"$hashScrnVars{desc}\" <$hashScrnVars{pkey}>";
    }
    if ($sipiaxfriend =~ /username=/) {
    		$sipiaxfriend =~ s/username=/username=$hashScrnVars{desc}/;
    }
    else {
		$sipiaxfriend .= "\nusername=$hashScrnVars{desc}";
    }
    if ($sipiaxfriend =~ /secret=/) {
    		$sipiaxfriend =~ s/secret=/secret=$hashScrnVars{passwd}/;
    }
    else {
		$sipiaxfriend .= "\nsecret=$hashScrnVars{passwd}";
    }
    if ($sipiaxfriend =~ /mailbox=/) {
		$sipiaxfriend =~ s/mailbox=/mailbox=$hashScrnVars{pkey}/;
    }
    else {
		$sipiaxfriend .= "\nmailbox=$hashScrnVars{pkey}";
    }
    if ($sipiaxfriend =~ /pickupgroup=/) {
		$sipiaxfriend =~ s/pickupgroup=/pickupgroup=1 \ncallgroup=1/;
    }
    else {
		$sipiaxfriend .= "\npickupgroup=1\ncallgroup=1";
    }
    unless ($sipiaxfriend =~ /call-limit/) {
                $sipiaxfriend .= "\ncall-limit=3";
    }
    unless ($sipiaxfriend =~ /subscribecontext/) {
                $sipiaxfriend .= "\nsubscribecontext=extensions";
    }
#    unless (db_get_prop(\%selintra, 'global', 'LACL') eq 'NO') {
	if ($acl eq 'YES') {
    	unless ($hashScrnVars{location} eq 'remote') {
        	unless ($sipiaxfriend =~ /deny=/) {
                	$sipiaxfriend .= "\ndeny=0.0.0.0/0.0.0.0";
                }
                unless ($sipiaxfriend =~ /permit=/) {
                        $sipiaxfriend .= "\npermit=$ipaddr/255.255.255.0";
                }
        }
    }
    $sipiaxfriend .= "\ndisallow=all ";

	if ( -e "/opt/sark/.astrelease") {
		$sipiaxfriend .= "\nallow=G722 ";
	}
    $sipiaxfriend .= "\nallow=alaw\nallow=ulaw ";
    if ($hashScrnVars{'location'} eq 'remote') {
	    $sipiaxfriend .= "\nnat=yes";
    }

    $sipiaxfriend =~ s/^\s+//;
    $sipiaxfriend =~ s/\s+$//;
    $sipiaxfriend =~ s/\r//g;

    $provision =~ s/^\s+//;
    $provision =~ s/\s+$//;
    $provision =~ s/\r//g;
    $provision =~ s//|/g;

    $hashScrnVars{sipiaxfriend} =  $sipiaxfriend;



# SPA 2x SUPPORT
#	my $device2 = '';
#	my $macaddr2 = '';
#	my $linenum = '';
#	my $count = 0;
#        my @IPphone = ();
#
#	if ($device =~ /SPA-2\d{3}/) {
#    	    foreach (sort keys %selintra) {
#      	    	push (@IPphone, $_)
#        	    if (db_get_type(\%selintra, $_) eq "IPphone");
#    	    }
#
#            foreach (@IPphone) {
#            	$device2   = db_get_prop(\%selintra, $_, 'device')   || '';
#            	$macaddr2  = db_get_prop(\%selintra, $_, 'macaddr')  || '';
#            	$linenum   = db_get_prop(\%selintra, $_, 'linenum')  || '';
#            	if ($device2 =~ /SPA-2\d{3}/ && $macaddr eq $macaddr2) {
#                    $count++;
#                }
#            }
#            if ($count == 1) {
#                $linenum = "1";
#            }
#            elsif ($count == 2) {
#		if ($linenum eq '2') {
#		    $linenum = "1";
#		}
#		else {
#                    $linenum = "2";
#		}
#            }
#            db_set_prop(\%selintra, $rule, 'linenum', $linenum);
#	    $provision =~ s/_x_/_1_/g;
#	    if ($linenum eq '2') {
#		$provision =~ s/_1_/_2_/g;
#	    }
#        }

	$provision =~ s/\$desc/$hashScrnVars{desc}/g;
	$provision =~ s/\$ext/$hashScrnVars{pkey}/g;
	$provision =~ s/\$password/$hashScrnVars{passwd}/g;
#	if ($location eq 'remote' && $externalip ne '') {
#	    $provision =~ s/\$localip/$externalip/g;
#	}
#	else {
# 	    $provision =~ s/\$localip/$ipaddr/g;
#	}

#	$provision =~ s/\|//g;
#     	db_set_prop(\%selintra, $rule, "provision", $provision);
	$provision =~ s//|/g;
        $hashScrnVars{provision} =  $provision;
#	my $fmacaddr = db_get_prop(\%selintra, $rule, "macaddr")    || "";
        my $dbh = SarkSubs::SQLiteConnect();
        my %hashRecordColumns = SarkSubs::SQLiteGetHashCols($dbh, 'IPphone');
        SarkSubs::SQLiteDisconnect($dbh);
        $gonogo = SarkSubs::validateTuple(\%hashScrnVars, \%hashRecordColumns, \%hashVarRegexs,
    			\%hashVarOutBuf, \%hashColorDefaults,
                        \@Verrors);
#        createValidate();
        unless ($gonogo) {
#
#		Set defaults
#
        	while ( my ($key, $value) = each %hashVarDefaults ) {
                	unless ( $hashVarOutBuf{$key} ) {
                        	if ($hashVarDefaults{$key}) {
                        		$hashVarOutBuf{$key} = $hashVarDefaults{$key};
                                }
                        }
                }
#
#		Create record
#
        	my $dbh = SarkSubs::SQLiteConnect();
#
#               doCOS
#
		foreach (@Cos) {
                	if (SarkSubs::SQLiteGet($dbh, "SELECT defaultopen FROM COS where pkey = '$_'") eq "YES" ) {
				SarkSubs::SQLiteDo($dbh, "INSERT INTO IPphoneCOSopen  (IPphone_pkey,COS_pkey) VALUES ('$hashScrnVars{pkey}','$_')" );
                        }
                        if (SarkSubs::SQLiteGet($dbh, "SELECT defaultclosed FROM COS where pkey = '$_'") eq "YES" ) {
				SarkSubs::SQLiteDo($dbh, "INSERT INTO IPphoneCOSclosed  (IPphone_pkey,COS_pkey) VALUES ('$hashScrnVars{pkey}','$_')" );
                        }
                }
#
#               New Record
#
        	SarkSubs::newTuple( \%hashVarOutBuf, "IPPhone", $dbh);

                SarkSubs::SQLiteDisconnect($dbh);
                if ($q->param('newcommit.x') ) {
                	SarkSubs::sysCommit($q);
                }
                showInitial ($q,"Endpoint created!");
    	}
        else {
        	createIPphone ($q);
        }
        return 0;
}

sub linesIn ($)
{
    my ($q) = shift;

    my $pkey     = $q->param ('pkey');
    my $delete   = $q->param ('delete');
    my $notify   = $q->param ('notify');
    my $change   = $q->param ('change');
    my $srchkey  = $q->param ('srchkey');

    if ($delete eq 'yes') {
    	performDelete ($q);
    	return 0;
    }
    if ($notify eq 'yes') {
    	performNotify ($q);
    	return 0;
    }

    my $dbh = SarkSubs::SQLiteConnect();

    if ($srchkey) {
	$pkey = SarkSubs::SQLiteGet($dbh, "SELECT pkey FROM IPphone where pkey = '$srchkey'");
	unless ($pkey) {
		my $msg = "Extension invalid (enter number only)";
		showInitial ($q, $msg);
		exit 0;
       }
   }

    my %IPphone    = SarkSubs::SQLiteGetHash($dbh, "SELECT * FROM IPphone WHERE pkey = '$pkey'");
    my $commit = SarkSubs::SQLiteGet($dbh, "SELECT MYCOMMIT FROM globals where pkey = 'global'");
    my $cosstart = SarkSubs::SQLiteGet($dbh, "SELECT COSSTART FROM globals where pkey = 'global'");
    my @Open;
    my @Closed;
    my @IPphones = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM IPphone" ));
    my @Clusters = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM Cluster"));
    my @Trunks = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM lineIO"));
    my @Speed = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM speed"));
    my @Capp = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM Appl"));
    my @Cos = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM COS"));
    my @IVR = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM ivrmenu"));
    foreach (@Cos) {
    	push (@Open, $_)
        	if (SarkSubs::SQLiteGet($dbh, "SELECT IPphone_pkey FROM IPphoneCOSopen where IPphone_pkey = '$pkey' and COS_pkey = '$_'"));
        push (@Closed, $_)
        	if (SarkSubs::SQLiteGet($dbh, "SELECT IPphone_pkey FROM IPphoneCOSclosed where IPphone_pkey = '$pkey' and COS_pkey = '$_'"));
    }
    
    

#
#  Log in to the Asterisk manager and retrieve CFWD info
#
    my $astman = Asterisk::AMI::Common->new(PeerAddr        =>      '127.0.0.1',
                                                PeerPort        =>      '5038',
                                                Username        =>      'sark',
                                                Secret          =>      'mysark'
                                );
    my $cfim;
    my $cfbs;
    my $ringdelay;
    if ($astman) {$cfim = $astman->db_get('cfim', $pkey) || "NOTSET";}
    if ($astman) {$cfbs = $astman->db_get('cfbs', $pkey) || "NOTSET";}
    if ($astman) {$ringdelay = $astman->db_get('ringdelay', $pkey) }
    if ($ringdelay eq "") {
		$ringdelay = "DEFAULT";
	}
		
#
#   Build Cross References
#

    my $xref;
    my $tref;
    foreach (@IPphones) {
    	unless ($_ eq $pkey) {
        	my %rec    = SarkSubs::SQLiteGetHash($dbh, "SELECT * FROM IPphone WHERE pkey = '$_'");
        	if ( $rec{dvrvmail} eq $pkey ) {
                	$tref .= "This extension provides a voicemailbox for  $_<br>";
        	}
        }
    }
    if ($tref) {
    	$xref .= $tref;
        $tref = "";
    }
    else {
    	$xref .= "No other extensions reference this extension's voicemail box<br/>";
    }
    foreach (@Trunks) {
        my %rec = SarkSubs::SQLiteGetHash($dbh, "SELECT * FROM lineIO WHERE pkey = '$_'");
        if ( $rec{openroute} eq $pkey || $rec{closeroute} eq $pkey ) {
                $tref .= "Trunk $_ references this extension <br>";
        }

    }
    if ($tref) {
    	$xref .= $tref;
        $tref = "";
    }
    else {
    	$xref .= "No Trunks reference this extension <br/>";
    }
    foreach (@Speed) {
        my %rec = SarkSubs::SQLiteGetHash($dbh, "SELECT * FROM speed WHERE pkey = '$_'");
        if ( $rec{out} =~ $pkey || $rec{outcome} =~ $pkey ) {
                /^(\d{1,4})/;
                $tref .= "Callgroup $1 references this extension <br>";
        }
    }
    if ($tref) {
    	$xref .= $tref;
        $tref = "";
    }
    else {
    	$xref .= "No Callgroups reference this extension <br/>";
    }
    foreach (@Capp) {
        my %rec = SarkSubs::SQLiteGetHash($dbh, "SELECT * FROM Appl WHERE pkey = '$_'");
        if ( $rec{extcode} =~ $pkey ) {
                $tref .= "Custom App $_ <i>may</i> reference this extension in its code <br>";
        }
    }
    if ($tref) {
    	$xref .= $tref;
        $tref = "";
    }
    else {
    	$xref .= "No CustomApps reference this extension<br/>";
    }
    foreach (@IVR) {
        my %rec = SarkSubs::SQLiteGetHash($dbh, "SELECT * FROM ivrmenu WHERE pkey = '$_'");
        if ($rec{timeout} eq $pkey ) {
                $tref .= "IVR Timeout $_ references this extension <br>";
        }
        else {
                for ( my $i=0; $i <= 11; $i++ ) {
                        if  ($rec{"option".$i} eq $pkey) {
                                $tref .=  "IVR $_ references this extension <br>";
                                last;
                        }
                }
        }
    }
    if ($tref) {
    	$xref .= $tref;
        $tref = "";
    }
    else {
    	$xref .= "No IVRs reference this extension <br/>";
    }

    # untaint the data
    $pkey =~ /^(\d+)$/;
    $pkey = $1;

    SarkSubs::sark_header ($q);
    my $printline = " $IPphone{technology}/$IPphone{pkey}, User \"$IPphone{desc}\"($IPphone{device})";

    print h1( $printline );

    print '<hr class="hr" />';

    print $q->hidden (-name => 'pkey',  -override => 1, -default => $pkey);
    if (@Verrors) {
	SarkSubs::sark_error(\@Verrors);
    }
    print '<div class="buttons">';
        SarkSubs::sark_button_cancel();
    	SarkSubs::sark_button_save("upsave");
        SarkSubs::sark_button_commit( "commit", $commit);
		unless ($IPphone{technology} eq 'Analogue' || $IPphone{technology} eq 'DAHDI' || $IPphone{technology} eq 'Zap')  {
			SarkSubs::sark_button_delete('delete');
		}
    print '</div>';

    print '<div id="pagetabs"  class="ui-tabs">';
    print "\n";
    print '<ul>'."\n";
    print '<li><a href="#tab-1">General</a></li>'."\n";

    unless ($IPphone{technology} eq 'Analogue' || $IPphone{technology} eq 'Custom' )  {
   	print  '<li><a href="#tab-2">Asterisk</a></li>'."\n";
	print  '<li><a href="#tab-3">Provisioning</a></li>'."\n";
    }
    if ( @Cos ) {
		print '<li><a href="#cos" >CoS</a></li>'. "\n";
	}
    print '<li><a href="#cfwd" >CFWD</a></li>'."\n";
    print '<li><a href="#xref" >XREF</a></li>'."\n";
    print '</ul>'."\n";

#
#   TAB Provisioning
#
    unless ($IPphone{technology} eq 'Analogue' || $IPphone{technology} eq 'Custom' )  {
    	print '<div id="tab-3" class="ui-tabs-hide">';

        	print $q->textarea(
                	-class    =>'sipiax',
        		-name	  =>'provision',
                	-default  =>$IPphone{provision},
                	-rows     =>15,
                	-columns  =>100);
#
#       TAB DIVEND
#
        print '</div>';
    }
#
#   TAB Asterisk
#
    unless ($IPphone{technology} eq 'Analogue' || $IPphone{technology} eq 'Custom' )  {
    	print '<div id="tab-2" class="ui-tabs-hide" >';
        	print $q->textarea(
                	-class    =>'sipiax',
        		-name	  =>'sipiaxfriend',
                	-default  =>$IPphone{sipiaxfriend},
                	-rows     =>15,
                	-columns  =>100);
#
#       TAB DIVEND
#
        print '</div>';
    }
#
#   TAB General
#
    print '<div id="tab-1" class="ui-tabs-hide" >';
    print SarkSubs::aLabelFor('rule');
    print  $q->textfield (
                	-name     => 'displaypkey',
                	-override => 1,
                        -readonly =>'readonly',
                        -style    =>'background-color: lightgrey',
                	-default  => $pkey,
        		-size     => 4);

     unless  ( $IPphone{device} =~ /VXT/ )  {
          print SarkSubs::aLabelFor('macaddr');
     	  print  $q->textfield (
                	-name     => 'macaddr',
                	-override => 1,
                        -style    => "color: $hashColorDefaults{macaddr}",
                	-default  => $IPphone{macaddr},
        		-size     => 16);
     }
     else {
          print SarkSubs::aLabelFor('macaddr');
     	  print  $q->textfield (
                	-name     => 'macaddr',
                	-override => 1,
                        -readonly =>'readonly',
                        -style    =>'background-color: lightgrey',
                	-default  => $IPphone{macaddr},
        		-size     => 16);
     }

     print SarkSubs::aLabelFor('vmailfwd');
     print  $q->textfield (
                	-name     => 'vmailfwd',
                	-override => 1,
                        -style    => "color: $hashColorDefaults{vmailfwd}",
                	-default  => $IPphone{vmailfwd},
        		-size     => 20);

     print SarkSubs::aLabelFor('cluster');
     print $q->popup_menu (
                     -name    => 'cluster',
                     -values  => [@Clusters],
                     -default => $IPphone{cluster});

     print SarkSubs::aLabelFor('devicerec');
     print $q->popup_menu (
                     -name    => 'devicerec',
                     -values  => [SarkSubs::get_rec_list,'Inbound','Outbound','Both'],
                     -default => $IPphone{devicerec});

     unless ($IPphone{technology} eq 'Analogue' || $IPphone{technology} eq 'Custom' )  {
     	print SarkSubs::aLabelFor('location');
    	print $q->popup_menu (
                     -name    => 'location',
                     -values  => ['local', 'remote'],
                     -default => $IPphone{location});
     }

     print SarkSubs::aLabelFor('dvrvmail');
     print $q->popup_menu (
                     -name    => 'dvrvmail',
                     -values  => [$pkey, "None", @IPphones],
                     -default => $IPphone{dvrvmail});

     print SarkSubs::aLabelFor('callerid');
     print  $q->textfield (
                	-name     => 'callerid',
                	-override => 1,
                        -style    => "color: $hashColorDefaults{callerid}",
                	-default  => $IPphone{callerid},
        		-size     => 12);
     unless ($IPphone{technology} eq 'Analogue' || $IPphone{technology} eq 'Custom' )  {
     	print SarkSubs::aLabelFor('extalert');
     	print  $q->textfield (
                        -name     => 'extalert',
                        -override => 1,
                        -style    => "color: $hashColorDefaults{extalert}",
                        -default  => $IPphone{extalert},
                        -size     => 30);

     }
     if ( $IPphone{technology} eq "Custom" ) {
    	print SarkSubs::aLabelFor('cdialstring');
    	print $q->textfield (
                    -name     => 'dialstring',
                    -override => 1,
                    -default  => $IPphone{dialstring},
                    -size     => 30);
    }

#
#       TAB DIVEND
#
        print '</div>';
#
#   TAB Class of Service
#

  if ( @Cos ) {
    print '<div id="cos" class="ui-tabs-hide" >';
    print "\n";
    print $q->br;
    print $q->br;
    print $q->br;

    print '<span style="color: rgb(0, 0, 0); font-weight:bold; font-size:small; ">Day-time Class of Service</span><br />';
    foreach (@Cos) {
       	if (SarkSubs::SQLiteGet($dbh, "SELECT IPphone_pkey FROM IPphoneCOSopen where IPphone_pkey = '$pkey' and COS_pkey = '$_'")) {
    		print "<input type=\"checkbox\" checked=\"yes\" name=\"opencos\" value=\"$_\" />$_<br />\n";
        }
        else {
                print "<input type=\"checkbox\" name=\"opencos\" value=\"$_\" />$_<br />\n";
        }
    }
    print '<br/><br/><span style="color: rgb(0, 0, 0); font-weight:bold; font-size:small; ">Night-time Class of Service</span><br />';
    foreach (@Cos) {
       	if (SarkSubs::SQLiteGet($dbh, "SELECT IPphone_pkey FROM IPphoneCOSclosed where IPphone_pkey = '$pkey' and COS_pkey = '$_'")) {
    		print "<input type=\"checkbox\" checked=\"yes\" name=\"closedcos\" value=\"$_\" />$_<br />\n";
        }
        else {
                print "<input type=\"checkbox\" name=\"closedcos\" value=\"$_\" />$_<br />\n";
        }
    }
    print "\n";
    print '</div>';
  }
#
#       TAB DIVEND
#

#
#   TAB Call Forwards
#
    print '<div id="cfwd" class="ui-tabs-hide">';
    print "\n";
    print $q->br;
    print $q->br;
    print $q->br;
    print '<span style="color: rgb(0, 0, 0); font-weight:bold; font-size:small; ">Call Forwards</span><br />';
    print $q->br;
    print SarkSubs::aLabelFor('cfim');
    print  $q->textfield (
                        -name     => 'cfim',
                        -override => 1,
                        -default  => $cfim,
                        -size     => 12);
#    print '<br/><br/><span style="color: rgb(0, 0, 0); font-weight:bold;font-size:small; ">Call forward on busy ';

    print SarkSubs::aLabelFor('cfbs');
    print  $q->textfield (
                        -name     => 'cfbs',
                        -override => 1,
                        -default  => $cfbs,
                        -size     => 12);
#    print '<br/><br/><span style="color: rgb(0, 0, 0); font-weight:bold; font-size:small; ">Ringdelay ';
    print $q->br;
    print '<span style="color: rgb(0, 0, 0); font-weight:bold; font-size:small; ">Ring Time</span><br />';
    print $q->br;
    print SarkSubs::aLabelFor('ringdelay');
    print  $q->textfield (
                        -name     => 'ringdelay',
                        -override => 1,
                        -default  => $ringdelay,
                        -size     => 10);

    print '</div>';
#
#       TAB DIVEND
#

#
#   TAB XREF table
#
    print '<div id="xref" class="ui-tabs-hide" >';
    print "\n";
    print '<span style="color: rgb(0, 0, 0); font-weight:bold; font-size:small; ">Cross References to this extension</span><br />';
    print $q->br;
    print "$xref";
    print $q->br;
    print '</div>';
#
#       TAB DIVEND
#



#
#  end of TABS DIV
#
     print '</div>';

    SarkSubs::SQLiteDisconnect($dbh);
    SarkSubs::sark_footer($q);
}


sub performLinesIn ($)
{
    my ($q) = shift;
    %hashScrnVars = $q->Vars ;

    my $response;
    my $dbh = SarkSubs::SQLiteConnect();
    my %IPphone = SarkSubs::SQLiteGetHash($dbh, "SELECT * FROM IPphone WHERE pkey = '$hashScrnVars{pkey}'");
    my @siplines = split /\n/, $hashScrnVars{sipiaxfriend};

    foreach ( @siplines ) {
	if (/username=(.*)/) {
		$hashScrnVars{desc} = $1;
	}
    }    

    if ($hashScrnVars{sipiaxfriend} =~ /nat=yes/) {
        if ($hashScrnVars{location} eq 'local') {
			$hashScrnVars{sipiaxfriend} =~ s/nat=yes//;
	 }
    }
    else {
        if ($hashScrnVars{location} eq 'remote') {
    		$hashScrnVars{sipiaxfriend} .= "\nnat=yes";
		}
		else {
			$hashScrnVars{sipiaxfriend} =~ s/nat=yes//;
		}
    }
    $hashScrnVars{sipiaxfriend} =~ s/\r//g;
    $hashScrnVars{provision} =~ s/\r//g;
    
    
    $gonogo = SarkSubs::validateTuple(\%hashScrnVars, \%IPphone, \%hashVarRegexs,
    			\%hashVarUpdBuf, \%hashColorDefaults,
                        \@Verrors);
    
    
    
    unless ( $gonogo ) {
    	SarkSubs::updateTuple (\%hashVarUpdBuf, \%IPphone, "IPphone", $dbh);
    }
    SarkSubs::SQLiteDisconnect($dbh);
# UPDATE ANY CALL FORWARDS
    unless ( $gonogo ) {
        my $astman = Asterisk::AMI::Common->new(PeerAddr        =>      '127.0.0.1',
                                                PeerPort        =>      '5038',
                                                Username        =>      'sark',
                                                Secret          =>      'mysark'
                                );

        foreach my $cfwd (qw/cfim cfimop cfimcl cfbs cfbsop cfbscl ringdelay/) {
           if (exists $hashScrnVars{$cfwd}) { 
        	if ($hashScrnVars{$cfwd} =~ /^(\d*)$/) {
                       if ($astman) {$astman->db_put($cfwd, $hashScrnVars{pkey}, $1 );}
                        $response =  "Entry Saved!";
        	}
        	else {
                        if ($astman) {$astman->db_put($cfwd, $hashScrnVars{pkey}, '' );}
        	}
           }
        }
    }
    if ( $gonogo ) {
    	linesIn($q);
    }
    else {
#
#	All is good so do the Booleans for COS.
#
    	doCOS($q);
        if ($q->param('commit.x')) {
                SarkSubs::sysCommit($q);
        }
    	showInitial ($q,"Updates applied" );
    }

    return 0;
}


sub doCOS($) {
# Do the Booleans
    my $q = shift;
    my $dbh = SarkSubs::SQLiteConnect();
    my $pkey = $q->param('pkey');

#    SarkSubs::SQLiteSetUndoTrigger($dbh,"IPphoneCOSopen");
#    SarkSubs::SQLiteSetUndoTrigger($dbh,"IPphoneCOSclosed");


    my @opencos = $q->param('opencos');
    my @closedcos = $q->param('closedcos');

    SarkSubs::SQLiteDo($dbh, "DELETE from IPphoneCOSopen WHERE IPphone_pkey = '$pkey'");
    SarkSubs::SQLiteDo($dbh, "DELETE from IPphoneCOSclosed WHERE IPphone_pkey = '$pkey'");

    foreach ($q->param('opencos')) {
    	unless (SarkSubs::SQLiteGet($dbh, "SELECT IPphone_pkey FROM IPphoneCOSopen where IPphone_pkey = '$pkey' and COS_pkey = '$_'")) {
               SarkSubs::SQLiteDo($dbh, "INSERT INTO IPphoneCOSopen  (IPphone_pkey,COS_pkey) VALUES ('$pkey','$_')" );
        }
    }
    foreach (@closedcos) {
    	unless (SarkSubs::SQLiteGet($dbh, "SELECT IPphone_pkey FROM IPphoneCOSclosed where IPphone_pkey = '$pkey' and COS_pkey = '$_'")) {
               SarkSubs::SQLiteDo($dbh, "INSERT INTO IPphoneCOSclosed  (IPphone_pkey,COS_pkey) VALUES ('$pkey','$_')" );
        }
    }

    SarkSubs::SQLiteDo($dbh, "UPDATE globals SET MYCOMMIT='YES' WHERE pkey='global'");
    SarkSubs::SQLiteCommit($dbh);
    SarkSubs::SQLiteDisconnect($dbh);
}

sub performDelete ($)
{
    	my ($q) = shift;
    	my $pkey = $q->param ('pkey');
        my $dbh = SarkSubs::SQLiteConnect();
        my $macaddr = SarkSubs::SQLiteGet($dbh, "SELECT macaddr FROM IPphone where pkey = '$pkey'");

	if ($macaddr) {
		my $lcmacaddr =  $macaddr;
		$lcmacaddr =~ tr/A-Z/a-z/;
		my @Provfiles = </tftpboot/*>;
		foreach (@Provfiles) {
	                if (  $_ =~ m{$lcmacaddr} || $_ =~ m{$macaddr} ) {
				/(.*)/;
               			unlink $1;
               		}
		}
        }


	# if extension has changed then change extension in rest of database
	# change operator in globals
	if (SarkSubs::SQLiteGet($dbh, "SELECT SYSOP FROM globals where pkey = 'global'") eq $pkey) {
             my $sipiaxstart = SarkSubs::SQLiteGet($dbh, "SELECT SIPIAXSTART FROM globals where pkey = 'global'");
	}
	# change inbound routes in trunks
	my @Trunk = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM lineIO" ));
	foreach (@Trunk) {
	    if (SarkSubs::SQLiteGet($dbh, "SELECT openroute FROM lineIO where pkey = '$_'") eq $pkey) {
		SarkSubs::SQLiteDo($dbh, "UPDATE lineIO SET openroute='Operator' WHERE pkey='$_'");
	    }
	    if (SarkSubs::SQLiteGet($dbh, "SELECT closeroute FROM lineIO where pkey = '$_'") eq $pkey) {
		SarkSubs::SQLiteDo($dbh, "UPDATE lineIO SET closeroute='Operator' WHERE pkey='$_'");
	    }
	}
	# change cluster operator
	my @Cluster = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM Cluster" ));
	foreach (@Cluster) {
	    if (SarkSubs::SQLiteGet($dbh, "SELECT operator FROM Cluster where pkey = '$_'") eq $pkey) {
                SarkSubs::SQLiteDo($dbh, "UPDATE Cluster SET operator='System Operator' WHERE pkey='$_'");
	    }
	}
	# change alias outcome and number directed to
	my @Speed = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM speed" ));
	foreach (@Speed) {
	    if (SarkSubs::SQLiteGet($dbh, "SELECT outcome FROM speed where pkey = '$_'") eq $pkey) {
		SarkSubs::SQLiteDo($dbh, "UPDATE speed SET outcome='Operator' WHERE pkey='$_'");
	    }
	    my $out = SarkSubs::SQLiteGet($dbh, "SELECT out FROM speed where pkey = '$_'");
	    $out =~ s/^$pkey$//;
	    $out =~ s/^$pkey //;
	    $out =~ s/ $pkey / /;
	    $out =~ s/ $pkey$//;
	    SarkSubs::SQLiteDo($dbh, "UPDATE speed SET out='$out' WHERE pkey='$_'");
	}
	# change greeting options
	my @IVR = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM speed" ));
        my $keypress;
        my $IVRkey;
	foreach (@IVR) {
                $IVRkey = $_;
        	foreach $keypress (qw/ option0 option1 option2 option3 option4
                			  option5 option6 option7 option8 option9
                                          option10 option11 timeout/) {
                	if (SarkSubs::SQLiteGet($dbh, "SELECT $keypress FROM ivrmenu where pkey = '$IVRkey'") eq $pkey) {
                        	SarkSubs::SQLiteDo($dbh, "UPDATE ivrmenu SET $keypress='None' WHERE pkey='$IVRkey'");
                        }
                }
        }

        SarkSubs::SQLiteDelete($dbh,"DELETE FROM IPphone where pkey='$pkey'");
        SarkSubs::SQLiteDelete($dbh,"DELETE FROM IPphoneCOSopen where IPphone_pkey='$pkey'");
        SarkSubs::SQLiteDelete($dbh,"DELETE FROM IPphoneCOSclosed where IPphone_pkey='$pkey'");
        SarkSubs::SQLiteDo($dbh, "UPDATE globals SET MYCOMMIT='YES' WHERE pkey='global'");
    	SarkSubs::SQLiteCommit($dbh);
    	SarkSubs::SQLiteDisconnect($dbh);
    	showInitial ($q, "<font color = red>Successfully deleted.</font color = red><font color = black>");
}

sub sark_mode ($) {
    my ($ext) = shift;
    if ( -e "/opt/sark/modeone") {
    	if ($ext > 4) {
    		return (-1);
        }
    }
    return ();
}

sub performNotify ($)
{
    	my ($q) = shift;
    	my $pkey = $q->param ('pkey');

        my $dbh = SarkSubs::SQLiteConnect();
		my %IPphone = SarkSubs::SQLiteGetHash($dbh, "SELECT * FROM IPphone WHERE pkey = '$pkey'");
		SarkSubs::SQLiteDisconnect($dbh);
		unless ($IPphone{technology} eq 'SIP') {
				showInitial ($q, "<font color = red>This is not a SIP UA!!.</font color = red><font color = black>");
				return;
		}
#
#	figure out what it is 
#
		my $chk;
		if ($IPphone{device} =~ /Aastra/) {
			$chk = 'aastra-check-cfg';
		}
		if ($IPphone{device} =~ /Linksys/) {
			$chk = 'sipura-check-cfg';
		}
		if ($IPphone{device} =~ /Cisco/) {
			$chk = 'cisco-check-cfg';
		}		
		if ($IPphone{device} =~ /Polycom/) {
			$chk = 'polycom-check-cfg';
		}	
		if ($IPphone{device} =~ /Snom/) {
			$chk = 'snom-reboot';
		}
		if ($IPphone{device} =~ /Yealink/) {
			$chk = 'yealink-reboot';
		}
		if ($IPphone{device} =~ /Grandstream/) {
			$chk = 'grandstream-check-cfg';
		}
		unless ( $chk ) {
			showInitial ($q, "<font color = red>No notify available for device $IPphone{device}.</font color = red><font color = black>");
			return;
		}
		if ( -e "/persistent" ) {
			SarkSubs::request_syscmd ("/bin/asterisk -rx 'sip notify $chk $pkey' ");
		}
		else {
			SarkSubs::request_syscmd ("/usr/sbin/asterisk -rx 'sip notify $chk $pkey' ");
    		}
    	showInitial ($q, "<font color = red>Issued SIP Notify to Ext $pkey ($IPphone{device}).</font color = red><font color = black>");
}
