#!/usr/bin/perl

#----------------------------------------------------------------------
# heading     : Telephony
# description : Trunklines
# navigation  : 4000 4804
#
# Copyright (c) Aelintra.com 2005-2010
#
#----------------------------------------------------------------------
#----------------------------------------------------------------------
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
#
# Technical support for this program is available from Selintra Ltd
# Please visit our web site www.selintra.com/ for details.
#----------------------------------------------------------------------


use lib '/opt/sark/perl/modules';

use strict;
use CGI ':all';
use CGI::Carp qw(fatalsToBrowser);
use sark::SarkSubs;
#use Net::Domain qw(hostname hostfqdn hostdomain);
use Asterisk::AMI::Common;
use URI::Escape;
sub showInitial ($$);
sub commit ($);
sub linesIn ($);
sub performLinesIn ($);
sub performCreate ($);
sub performGetKey ($);
sub performCreateIn ($);
sub performDelete ($);



BEGIN
{
    # Clear PATH and related environment variables so that calls to
    # external programs do not cause results to be tainted. See
    # "perlsec" manual page for details.

    $ENV {'PATH'} = '';
    $ENV {'SHELL'} = '/bin/bash';
    delete $ENV {'ENV'};
}

#esmith::util::setRealToEffective ();
$< = $>;
$( = $);

$CGI::POST_MAX=1024 * 100;  # max 100K posts
$CGI::DISABLE_UPLOADS = 1;  # no uploads


my $editicon = "<img src=\"/sark-common/edit.png\" border=0 title = \"@{[SarkSubs::tt_display('Sarkedittext')]}\">";
my $yesicon = "<img src=\"/sark-common/connected.png\" border=0 title = \"@{[SarkSubs::tt_display('Sarktinfotext')]}\">";
my $noicon = "<img src=\"/sark-common/actions/no.png\" border=0 title = \"@{[SarkSubs::tt_display('Sarkoffline')]}\" >";
my $warnicon = "<img src=\"/sark-common/warning.png\" border=0 title = \"@{[SarkSubs::tt_display('Sarkwarntext')]}\" >";
my $infoicon = "<img src=\"/sark-common/info.png\" border=0 >";
my $greenicon = "<img src=\"/sark-common/actions/apply.png\" border=0 title = \"@{[SarkSubs::tt_display('Sarkremotetext')]}\" >";
my $activeicon = "<img src=\"/sark-common/actions/apply.png\" border=0 title = \"@{[SarkSubs::tt_display('Sarkactivetext')]}\" >";
my $monicon = "<img src=\"/sark-common/actions/apply.png\" border=0 title = \"@{[SarkSubs::tt_display('Sarkmontext')]}\" >";
my $trashicon = "<img src=\"/sark-common/edittrash.png\" border=0 onclick=\"return confirmOK('Delete this Object - Confirm?')\" title = \"@{[SarkSubs::tt_display('Sarkdeletetext')]}\" >";

my %hashScrnVars;
my @Verrors;

my $gonogo = 0;
my %hashVarDefaults =(
        "active" 	=> "YES",
	"alertinfo" 	=> undef,
	"callerid" 	=> undef,
	"callprogress"  => "NO",
        "carrier" 	=> undef,
	"closeroute" 	=> "Operator",
	"cluster" 	=> "default",
	"desc" 		=> undef,
	"disapass" 	=> undef,
	"faxdetect"	=> "NO",
	"forceivr" 	=> undef,
        "host" 		=> undef,
	"inprefix" 	=> undef,
	"lcl" 		=> "NO",
	"macaddr"	=> undef,
	"match" 	=> undef,
	"moh" 		=> "NO",
	"monitor" 	=> "NO",
	"openfirewall" 	=> undef,
	"openroute" 	=> "Operator",
	"password" 	=> undef,
	"peername" 	=> undef,
        "pkey" 		=> undef,
	"postdial" 	=> undef,
	"predial" 	=> undef,
	"privileged" 	=> undef,
	"provision" 	=> undef,
	"register" 	=> undef,
	"routeable" => "NO",
        "routeclassopen" => 100,
        "routeclassclosed" => 100,
	"sipiaxpeer" 	=> undef,
	"sipiaxuser" 	=> undef,
	"subnet" 	=> undef,
	"subnet1" 	=> undef,
	"subnet2" 	=> undef,
	"subnetstr" 	=> undef,
	"subnet1str" 	=> undef,
	"subnet2str" 	=> undef,
	"swoclip" 	=> "NO",
	"tag" 		=> undef,
	"transform"	=> undef,
	"trunkname"	=> undef,
	"username" 	=> undef,
        );
my %hashVarErrors;
my %hashVarOutBuf = (
        "active" 	=> undef,
	"alertinfo" 	=> undef,
	"callerid" 	=> undef,
	"callprogress"  => undef,
        "carrier" 	=> undef,
	"closeroute" 	=> undef,
	"cluster" 	=> undef,
	"desc" 		=> undef,
	"disapass" 	=> undef,
	"faxdetect"	=> undef,
	"forceivr" 	=> undef,
        "host" 		=> undef,
	"inprefix" 	=> undef,
	"lcl" 		=> undef,
	"macaddr"	=> undef,
	"match" 	=> undef,
	"moh" 		=> undef,
	"monitor" 	=> undef,
	"openfirewall" 	=> undef,
	"openroute" 	=> undef,
	"password" 	=> undef,
	"peername" 	=> undef,
        "pkey" 		=> undef,
	"postdial" 	=> undef,
	"predial" 	=> undef,
	"privileged" 	=> undef,
	"provision" 	=> undef,
	"register" 	=> undef,
	"routeable" => undef,
        "routeclassopen" => undef,
        "routeclassclosed" => undef,
	"sipiaxpeer" 	=> undef,
	"sipiaxuser" 	=> undef,
	"subnet" 	=> undef,
	"subnet1" 	=> undef,
	"subnet2" 	=> undef,
	"subnetstr" 	=> undef,
	"subnet1str" 	=> undef,
	"subnet2str" 	=> undef,
	"swoclip" 	=> undef,
	"tag" 		=> undef,
	"transform"	=> undef,
        "trunkname"	=> undef,
	"username" 	=> undef,
        );
my %hashVarUpdBuf = ();
my %hashVarRegexs = (
    "AADEFAULT" => qr/(.*)/,
#	"alertinfo" => qr/^([\w\/-]{5,40})$/,
#	"callerid" 	=> qr/^(\d{3,18})$/,
	"callerid" 	=> qr/^([\dNXZ\[\]\.\*_-\s]+)$/,
	"desc" 		=> qr/^([\w\s~]{3,30})$/,
	"disapass" 	=> qr/^(\d{4})$/,
	"inprefix"	=> qr/^(\d{1,2})$/,
    "match"    	=> qr/^(\d\d)$/,
    "macaddr"  	=> qr/^([0-9a-fA-F:]{12,18})$/,
#   "provision"	=> qr/^([\w\\n\\r\-@=;]{,200})$/,
	"pkey" 		=> qr/^([\w\.\[\]\+\*~-]{3,30})$/,
    "provision" => qr/^(.*)$/ms,
#   "register" 	=> qr/^(\w{2,20}:\w{2.20}@\w{2,30}\/\w{2,20})$/,
    "sipiaxpeer" => qr/^(.*)$/ms,
    "sipiaxuser" => qr/^(.*)$/ms,
    "tag"      	=> qr/^(\w{2,20})$/,
    "transform" => qr/^([\d:\s]+)$/,
    "trunkname" => qr/^([\w\s-]{3,20})$/,
	"subnet" 	=> qr/(^([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])(\/(3[012]|[12]\d|\d))?$|^[\w-]+[\.\w]+$)/,
	"subnet1" 	=> qr/(^([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])(\/(3[012]|[12]\d|\d))?$|^[\w-]+[\.\w]+$)/,
	"subnet2" 	=> qr/(^([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])(\/(3[012]|[12]\d|\d))?$|^[\w-]+[\.\w]+$)/,
        );

my %hashColorDefaults = %hashVarDefaults;
while ( my $tk = each %hashColorDefaults ) {
        $hashColorDefaults{$tk} = 'black';
}
#my $account;
#my @account = db_get(\%accounts);

my $buttoncolour = 'lightskyblue';
my $maxundonum = '5';
my $systemName = `/bin/hostname` || 'sark';
$systemName =~ s/\s+$//;
my $ipaddr      = SarkSubs::ret_localip();
my $pages = 1 ;
my $page = 1;
my $epages = 4;
my $epage = 1;
my $plen = 12;
#------------------------------------------------------------
# Mainline
#------------------------------------------------------------

my $q = new CGI;
my %params = $q->Vars;
#
#  Retrieve the page number
#
for my $key ( keys %params ) {
       if ( $key =~ /^page(\d{1,}).x$/ ) {
       	  $page=$1;
          showInitial ($q, '');
          exit(0);
       }
       if ( $key =~ /^epage(\d{1,}).x$/ ) {
       	  $epage=$1;
          linesIn ($q);
          exit(0);
       }
}

if ($q->param ('new.x') )  {
	performGetKey ($q);
}
elsif ($q->param ('continue.x') ) {
	performCreate ($q);
}
elsif ($q->param ('cancel.x')) {
    showInitial ($q, '');
}
elsif ($q->param ('initcommit.x') ) {
        commit ($q);
}
elsif ($q->param ('commit.x') )  {
    performLinesIn ($q);
}

elsif ($q->param ('delete.x')) {
    performDelete ($q);
}
#elsif ($q->param ('undo.x'))
#{
#    SarkSubs::SQLiteUndo();
#    $q->delete_all();
#    showInitial ($q,"Undo Processing complete");
#}
elsif ($q->param ('carriertype') eq "Sibling") {
    createSibling ($q);
}
elsif ($q->param ('carriertype') eq "Spa3xxx") {
    createSpa ($q);
}
elsif ($q->param ('carriertype') eq "group" || $q->param ('carriertype') eq "Custom") {
    createGroup ($q);
}
elsif ($q->param ('carriertype') eq "DiD") {
    createDid ($q);
}
elsif ($q->param ('carriertype') eq "VOIP") {
    createVoip ($q);
}
elsif ($q->param ('carriertype') eq "CLID") {
    createClid ($q);
}
elsif ($q->param ('carriertype') eq "Class") {
    createClass ($q);
}
elsif ($q->param ('srchkey') ) {
    linesIn ($q);
}
elsif (! grep (/^state$/, $q->param)) {
    showInitial ($q, '');
}
elsif ($q->param ('state') eq "linesIn") {
    linesIn ($q);
}

elsif ($q->param ('state') eq "performLinesIn") {
    performLinesIn ($q);
}
#elsif ($q->param ('state') eq "performDelete") {
#    performDelete ($q);
#}
else {
#    esmith::cgi::genStateError ($q, \%conf);
     die "STATE ERROR!!  - ending \n";
}

exit (0);

#------------------------------------------------------------
# subroutine to display initial form
#------------------------------------------------------------

sub showInitial ($$)
{
    my ($q, $msg) = @_;
    my $pkey;
#    my $commit = db_get_prop(\%selintra, 'global', 'COMMIT')       || 'YES';
    my $rowcolor = 0x01;

    my $dbh = SarkSubs::SQLiteConnect();
    my $commit = SarkSubs::SQLiteGet($dbh, "SELECT MYCOMMIT FROM globals where pkey = 'global'");
    my @Lines = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM lineIO" ));
    my @Carriers = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM Carrier"));
    my $astman;
# astman doesn't always work first time - so we retry
    for (my $i=1; $i<3; $i++) {
		$astman = Asterisk::AMI::Common->new(
        PeerAddr        =>      '127.0.0.1',
    	PeerPort        =>      '5038',
        Username        =>      'sark',
        Secret          =>      'mysark'
        );
        if ($astman) {
			last;
		}
	}
			
#        die "Unable to connect to asterisk" unless ($astman);

    my $numlines = @Lines;
    SarkSubs::sark_header ($q);
    my $remainder = $numlines % $plen;
    $pages = int($numlines / $plen);
    if ($remainder) {
    	$pages++;
    }
    my $linelow = (($page-1) * $plen);
    my $linehigh = $linelow + $plen;

    my $printline =  SarkSubs::tt_display('Sarksystemh11').($msg || 'Trunklines');
    unless ($astman ) {
       $printline = SarkSubs::tt_display('Sarksystemh11')."Cannot retrieve live values from Asterisk";
    }
    print h1( $printline );
    print '<hr class="hr" />';
    print '<div class="buttons">';
    SarkSubs::sark_button_new();
#    if (SarkSubs::SQLiteCheckUndo) {
#        SarkSubs::sark_button_undo();
#    }
#    if ($commit eq 'YES') {
        SarkSubs::sark_button_commit("initcommit",$commit);
#    }
    print "</div>";



    print '<div>';
    print "<div class=etabs>";
    unless (@Lines) {
        print '<span style="color:red;font-weight:bold">'.SarkSubs::tt_display('Sarktrunkmsg1').'</span>';
        print '</div>';
        SarkSubs::sark_footer($q);
        SarkSubs::SQLiteDisconnect($dbh);
        exit(0);
    }
    print $q->table ({border => 1, cellspacing => 1, cellpadding => 0});
#        print $q->table;
    print $q->Tr (
                      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('line')),
                      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('cluster')),
                      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('trunkname')),
                      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('carriertype')),
                      
                      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('ipaddr')),
                      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('latency')),
#                      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('preselect')),
                      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('openroute')),
                      $q->th({class=>"srk-border"}, SarkSubs::aHeaderFor('closeroute')),
                      $q->th({class=>"srk-border"}, ''),
                      $q->th({class=>"srk-border"}, ''),
                      $q->th({class=>"srk-border"}, ''),
                      $q->th({class=>"srk-border"}, '')
        );


    my $iaxaction = "N/A";
    my @iax_peer;
    if ($astman) {
        	$iaxaction = $astman->send_action ({ 	Action => 'Command',
                                                Command => 'iax2 show peers'
                                          });
                my $response2 = $astman->get_response($iaxaction);
                @iax_peer = @{$response2->{CMD}};
    }

#    my $i = $linelow;;
#    while ($i<$linehigh && $i <= $#Lines) {
#            my $pkey  = @Lines[$i];
    foreach (@Lines) {
    	    $pkey = $_;
    	    my $escpkey = uri_escape($pkey);
            my %lineIO = SarkSubs::SQLiteGetHash($dbh, "SELECT * FROM lineIO WHERE pkey = '$_'");
#            my $carrier    	=($lineIO{'carrier'});
            my $technology 	= SarkSubs::SQLiteGet($dbh, "SELECT technology FROM Carrier where pkey = '$lineIO{carrier}'");
            my $carriertype     = SarkSubs::SQLiteGet($dbh, "SELECT carriertype FROM Carrier where pkey = '$lineIO{carrier}'");
            my $match		=($lineIO{'match'}) || 'None';
	    my $port;
	    my $peerstate = "N/A";
	    my $hostip;
        my $latency = "N/A";
	    if ($match =~ /(\d{1,2})/ || '') {
	    	$match = $1;
	    }
	    my $activeicon = $activeicon;
	    unless ($lineIO{'active'} eq "YES") {
	    	$activeicon = $noicon;
	    }
            my $monitoricon = $monicon;
            unless ($lineIO{'monitor'} eq "YES") {
               	$monitoricon = "";
            }
	    $peerstate = $noicon;
	    unless ($technology eq "SIP" || $technology eq "IAX2" )  {
	    	$peerstate = "N/A";
                $hostip = "LOCAL";
                $latency = "N/A";
                $port =  "N/A";
	    }
	    elsif ($technology eq "SIP")  {
            	if ($astman) {
		    if ( defined $astman->sip_peers() ) {
                	$latency = $astman->sip_peers()->{$lineIO{peername}}{Status};
                        $hostip = $astman->sip_peers()->{$lineIO{peername}}{IPaddress};
                	$port = $astman->sip_peers()->{$lineIO{peername}}{IPport};
		    }
                }
                if ($latency =~ /OK/ ) {
            		$latency =~ /^OK \((.*)\)$/;
                	$latency = $1;
                	$peerstate = $greenicon;
		}
		else {
			$peerstate = $noicon;
		}

            	if ($port eq "0") {
            		$port = "";
            	}
            }
	    elsif ($technology eq "IAX2") {
	    	foreach (@iax_peer) {
			if (/$lineIO{'peername'}/) {
                        	if ( /(\d{1,}\sms)/ || /(UNREACHABLE)/ || /(UNKNOWN)/ ) {
                                	$latency = $1;
                                }
                                if ( /(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/ ) {
                                       	$hostip = $1;
                                }
                                if ( /\s(\d{4,5})/ ) {
					$port = $1;
                                }
				if ( /OK/ ) {
					$peerstate = $yesicon;
				}
				last;
			}
		}
	    }
	    if ($latency eq 'UNREACHABLE') {
			$latency = 'U/R'
		}
	    if ($latency eq 'UNKNOWN') {
			$latency = 'UNK'
		}		
#            s/\*//;
	    if ( length $lineIO{'cluster'} > 9 ) {
			$lineIO{'cluster'} = substr($lineIO{'cluster'} , 0, 8);
			$lineIO{'cluster'} .= '.';
        }
#            unless ($lineIO{'trunkname'}) {
#            	$lineIO{'trunkname'} = 'Not named';
#            }
        if ( length $lineIO{trunkname} > 9 ) {
			$lineIO{trunkname} = substr($lineIO{trunkname} , 0, 8);
			$lineIO{trunkname} .= '.';
        }
        if ( length $lineIO{openroute} > 9 ) {
			$lineIO{openroute} = substr($lineIO{openroute} , 0, 8);
			$lineIO{openroute} .= '.';
        }
        if ( length $lineIO{closeroute} > 9 ) {
			$lineIO{closeroute} = substr($lineIO{closeroute} , 0, 8);
			$lineIO{closeroute} .= '.';
        }
            print $q->Tr ({-class => "d".$rowcolor},
                   $q->td({class=>"srk-border"}, $pkey),
                   $q->td({class=>"srk-border"}, $lineIO{'cluster'}),
                   $q->td({class=>"srk-border"}, $lineIO{'trunkname'}),
                   $q->td({class=>"srk-border"}, $carriertype),
					$q->td({class=>"srk-border"}, $hostip),
					$q->td({class=>"srk-border"}, $latency),
#		   $q->td({class=>"srk-border"}, $lineIO{'match'}),
					$q->td({class=>"srk-border"}, $lineIO{'openroute'}),
					$q->td({class=>"srk-border"}, $lineIO{'closeroute'}),
					$q->td({class=>"srk-border"}, $peerstate),
					$q->td({class=>"srk-border"}, $activeicon),
#                   $q->td({class=>"srk-border"}, $monitoricon),
					$q->td({class=>"srk-border"},
                    $q->a ({href => $q->url (-absolute => 1)
                        . "?state=linesIn&change=yes&pkey=$escpkey"},
			$editicon )),
                  $q->td({class=>"srk-border"},
                        $q->a ({href => $q->url (-absolute => 1)
                        . "?state=linesIn&delete=yes&pkey=$escpkey"},
			$trashicon )));
            $rowcolor = $rowcolor ^ 1;
#            $i++;
    }

    print '</table>';
    print '</div>';
#    SarkSubs::printpages($pages, $page);
#unless ($pages == 1) {
#    print <<SRCHKEY;
#<span style="position: relative; left: 30px; top: 8px; font-weight: bold; ">
#Line:
#<input type="text" name="srchkey" maxlength="15" size="12"  />
#</span>
#SRCHKEY
#}

    print '</div>';

    SarkSubs::sark_footer($q);
    SarkSubs::SQLiteDisconnect($dbh);

}

sub commit ($)
{
    SarkSubs::sysCommit($q);
    showInitial ($q, "<font color = red>@{[SarkSubs::tt_display('Sarksystemmsg1')]}</font color = red><font color = black>");
}

sub linesIn ($)
{
    my $q = shift;
    my $delete   = $q->param ('delete');
    my $change   = $q->param ('change');
    my $srchkey  = $q->param ('srchkey');

    if ($delete eq 'yes') {
    	performDelete ($q);
#        setupDelete($q);
    	return 0;
    }
    my $pkey =  uri_unescape($q->param ('pkey'));
	
    my $displaykey = $pkey;
#    print STDERR "PKEY IS $pkey\n";
#    $displaykey =~ s/\*//;

    my $dbh = SarkSubs::SQLiteConnect();

    if ($srchkey) {
	$pkey = SarkSubs::SQLiteGet($dbh, "SELECT pkey FROM lineIO where pkey = '$srchkey'");
	unless ($pkey) {
		my $msg = "Trunk key invalid";
		showInitial ($q, $msg);
		exit 0;
       }
   }

    my %lineIO = SarkSubs::SQLiteGetHash($dbh, "SELECT * FROM lineIO WHERE pkey = '$pkey'");

    my $carriertype     = SarkSubs::SQLiteGet($dbh, "SELECT carriertype FROM Carrier where pkey = '$lineIO{'carrier'}'")    || 'VOIP';
    my $technology      = SarkSubs::SQLiteGet($dbh, "SELECT technology FROM Carrier where pkey = '$lineIO{'carrier'}'");
    my @Conferences = ();
    my @IVRs = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM ivrmenu" ));
    push (@IVRs, "Default IVR");
    my @CustomApps = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM Appl" ));
    my @Clusters = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM Cluster"));
    my @Lines = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM lineIO" ));
    my @Extensions = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM IPphone" ));
    my @Carriers = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM Carrier"));
    my @Queues = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM Queue" ));
    my @SpeedDials = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM speed"));

#    unless ($technology eq 'SIP' || $technology =~ 'IAX2' || $lineIO{'carrier'} =~ /SPA-3\d{3}FXO/) {
#    	$epages = 2;
#    }

    foreach (@SpeedDials) {
    	s/speed//;
    }
    my @Vmails = ();
    foreach (@Extensions)  {
        push (@Vmails, "*".$_);
    }
    my @Trunks = ();
    my @Siblings = ();
    foreach (@Lines) {
        push (@Siblings, $_)
        	if (/\~/);
        my $cachecarrier = SarkSubs::SQLiteGet($dbh, "SELECT carrier FROM lineIO where pkey = '$_'");
        my $cachecarriertype = SarkSubs::SQLiteGet($dbh, "SELECT carriertype FROM Carrier where pkey = '$cachecarrier'");
        push (@Trunks, $_)
        	unless $cachecarriertype eq "DiD" ||
                	$cachecarriertype eq "CLID" ||
                        $cachecarriertype eq "Class" ||
                        $cachecarriertype eq "PSTN" ||
                        $lineIO{'pkey'} eq $_;
    }
    my $commit = SarkSubs::SQLiteGet($dbh, "SELECT MYCOMMIT FROM globals where pkey = 'global'");
    SarkSubs::SQLiteDisconnect($dbh);
    open IN, "</etc/asterisk/meetme.conf"
       	or die "Couldn't open meetme.conf  - ending \n";
    while (<IN>) {
      	unless (/;/) {
      		if (/^conf\s*=>\s*(\d{3,4})/) {
        		push (@Conferences, $1);
        	}
      	}
    }

    my $escape;
    my $numexists = 0;

    unless ($lineIO{'peername'}) {
    	$lineIO{'peername'} = $lineIO{'desc'};
    }
#
    my $openroute = $lineIO{'openroute'};
    if ($openroute =~ /^\*/) {
    	$escape = '\\'.$openroute;
        $openroute = $escape;
    }
    my $closeroute = $lineIO{'closeroute'};
    if ($closeroute =~ /^\*/) {
    	$escape = '\\'.$closeroute;
        $closeroute = $escape;
    }
    my $provision =  $lineIO{'provision'};
    $provision =~ s//|/g;

    SarkSubs::sark_header ($q);
    my $printline = "$lineIO{carrier}/$pkey Settings";

    print h1($printline);
    print '<hr class="hr" />';

    print $q->hidden (-name => 'pkey',  -override => 1, -default => $pkey);
    print $q->hidden (-name => 'state', -override => 1, -default => 'performLinesIn');
    if (@Verrors) {
	SarkSubs::sark_error(\@Verrors);
    }
    print '<div class="buttons">';
    	SarkSubs::sark_button_cancel();
    	SarkSubs::sark_button_save("save");
        SarkSubs::sark_button_commit( "commit",$commit);
   	unless ($technology eq 'Analogue' || $technology eq 'ISDNBRI' || $technology eq 'DAHDI' || $technology eq 'Zap')  {
    		SarkSubs::sark_button_delete("delete");
    	}
    print '</div>';


    print '<div id="pagetabs" class="ui-tabs" >';
    print "\n";
    print '<ul>'."\n";
    print '<li><a href="#route">Routing</a></li>'."\n";
    print '<li><a href="#line" >Line</a></li>'. "\n";
    if ($technology eq 'SIP' || $technology =~ 'IAX2' || $lineIO{'carrier'} =~ /SPA-3\d{3}FXO/) {
    	print '<li><a href="#peer" >Peer</a></li>'."\n";
    	print '<li><a href="#user" >User</a></li>'."\n";
    }
    print '</ul>'."\n";

#
#   TAB Route
#
    print '<div id="route" class="ui-tabs-hide">';

        print SarkSubs::aLabelFor('active');
        print $q->popup_menu (
                        -name    => 'active',
                        -values  => ['NO', 'YES'],
                        -default => $lineIO{active});

#        print SarkSubs::aLabelFor('carrier');
#        print $q->textfield (
#                	-name     => 'carrier',
#                	-default  => $lineIO{'carrier'},
#                        -readonly =>'readonly',
#                        -style    =>'background-color: lightgrey',
#        		-size     => 30);
#
        print SarkSubs::aLabelFor('cluster');
        print $q->popup_menu (
                        -name    => 'cluster',
                        -values  => [@Clusters],
                        -default => $lineIO{'cluster'});

        print SarkSubs::aLabelFor('faxonoff');
        print $q->popup_menu (
                       	-name    => 'faxdetect',
                       	-values  => ['NO', 'YES'],
                       	-default => $lineIO{faxdetect});
        unless ($technology eq "DiD" or $technology eq "Class") {
                  print SarkSubs::aLabelFor('match');
            	  print $q->textfield (
                                -name     => 'match',
                                -override => 1,
                                -style    => "color: $hashColorDefaults{match}",
                                -default  => $lineIO{'match'},
                                -size     => 2);
        }
        print SarkSubs::aLabelFor('swoclip');
        print $q->popup_menu (
                       	-name    => 'swoclip',
                       	-values  => ['NO', 'YES'],
                       	-default => $lineIO{swoclip});

        unless ($technology eq "DiD" || $technology eq "Class" || $pkey eq "Analog-In") {
               print SarkSubs::aLabelFor('transform');
               print $q->textfield (
                		-name     => 'transform',
                		-override => 1,
                        -style    => "color: $hashColorDefaults{transform}",
                		-default  => $lineIO{'transform'},
						-size     => 20);
        }
        unless ($carriertype eq "group" && $lineIO{routeable} ne "YES") {
                 print SarkSubs::aLabelFor('openroute');
                 print $q->popup_menu (
		  	-name    => 'openroute',
                  	-values  => ["Operator", "Hangup",
	                       optgroup (-name => "IVRs",
                               	         -values => [@IVRs]),
                               optgroup (-name => "QUEUES",
                                         -values => [@Queues]),
			       optgroup (-name => "EXTENSIONS",
					 -values => [@Extensions]),
			       optgroup (-name => "CALL GROUPS",
					 -values => [@SpeedDials]),
			       optgroup (-name => "DISA",
					 -values => ['DISA','CALLBACK']),
                               optgroup (-name => "GET VOICEMAIL",
                                         -values => ['Retrieve Voicemail']),
                               optgroup (-name => "LEAVE VOICEMAIL",
                                         -values => [@Vmails]),
			       optgroup (-name => "Siblings",
					 -values => [@Siblings]),
                               optgroup (-name => "Trunks",
					 -values => [@Trunks]),
			       optgroup (-name => "Custom Apps",
					 -values => [@CustomApps]),
                               optgroup (-name => "Conferences",
					 -values => [@Conferences])],
                  	-default => $openroute);

                 print SarkSubs::aLabelFor('closeroute');
                 print $q->popup_menu (
		  	-name    => 'closeroute',
                 	-values  => ["Operator", "Hangup",
	                       optgroup (-name => "IVRs",
                               	         -values => [@IVRs]),
                               optgroup (-name => "QUEUES",
                                         -values => [@Queues]),
			       optgroup (-name => "EXTENSIONS",
					 -values => [@Extensions]),
			       optgroup (-name => "CALL GROUPS",
					 -values => [@SpeedDials]),
			       optgroup (-name => "DISA",
					 -values => ['DISA','CALLBACK']),
                               optgroup (-name => "GET VOICEMAIL",
                                         -values => ['Retrieve Voicemail']),
                               optgroup (-name => "LEAVE VOICEMAIL",
                                         -values => [@Vmails]),
			       optgroup (-name => "Siblings",
					 -values => [@Siblings]),
                               optgroup (-name => "Trunks",
					 -values => [@Trunks]),
			       optgroup (-name => "Custom Apps",
					 -values => [@CustomApps]),
                               optgroup (-name => "Conferences",
					 -values => [@Conferences])],
                  	-default => $closeroute);
        }
        print SarkSubs::aLabelFor('trunkname');
        print $q->textfield (
               		-name     => 'trunkname',
               		-override => 1,
               		-default  => $lineIO{'trunkname'},
                        -style 	  => "Color: $hashColorDefaults{'trunkname'}",
          		-size     => 20);
#
#       TAB DIVEND
#
        print '</div>';
#    }
#
#   TAB Line
#
    	print '<div id="line" class="ui-tabs-hide">';

#    	print '<div class="buttons">';
#    		SarkSubs::sark_button_save("save");
#   		unless ($technology eq 'Analogue' || $technology eq 'ISDNBRI' || $technology eq 'DAHDI' || $technology eq 'Zap')  {
#    			SarkSubs::sark_button_delete("delete");
#    		}
#    		SarkSubs::sark_button_cancel();
#    		SarkSubs::sark_button_commit( "commit", $commit );
#    	print '</div>';

        print SarkSubs::aLabelFor('tag');
        print $q->textfield (
                		-name     => 'tag',
                		-override => 1,
                		-default  => $lineIO{'tag'},
                                -style 	  => "Color: $hashColorDefaults{'tag'}",
        			-size     => 20);

        print SarkSubs::aLabelFor('inprefix');
        print $q->textfield (
                                -name     => 'inprefix',
                                -override => 1,
                                -style    => "color: $hashColorDefaults{inprefix}",
                                -default  => $lineIO{inprefix},
                                -size     => 1);

        print SarkSubs::aLabelFor('moh');
        print $q->popup_menu (
                        	-name    => 'moh',
                        	-values  => ['NO', 'YES'],
                        	-default => $lineIO{moh});

        print SarkSubs::aLabelFor('disapass');
        print $q->textfield (
                                -name     => 'disapass',
                                -override => 1,
                                -style    => "color: $hashColorDefaults{disapass}",
                                -default  => $lineIO{disapass},
                                -size     => 4);

        unless ($technology eq "DiD" or $technology eq "Class") {
                  print SarkSubs::aLabelFor('callerid');
            	  print $q->textfield (
                                -name     => 'callerid',
                                -override => 1,
                                -style    => "color: $hashColorDefaults{callerid}",
                                -default  => $lineIO{'callerid'},
                                -size     => 12);
         }

         unless ($carriertype eq "group") {
                print SarkSubs::aLabelFor('alertinfo');
            	print $q->textfield (
                                -name     => 'alertinfo',
                                -override => 1,
                                -style    => "color: $hashColorDefaults{alertinfo}",
                                -default  => $lineIO{'alertinfo'},
                                -size     => 20);
         }

         unless ($carriertype eq 'Spa3xxx' || $carriertype eq 'Gateway') {
         	if ($technology eq "SIP" || $technology eq "IAX2") {
                        print SarkSubs::aLabelFor('register');
            		print $q->textfield (
                			-name     => 'register',
                	       		-override => 1,
                                        -style    => "color: $hashColorDefaults{register}",
                			-default  => $lineIO{'register'},
        				-size     => 30);

			print SarkSubs::aLabelFor('callprogress');
       			print $q->popup_menu (
                			-name     => 'callprogress',
                        		-values   => ['NO', 'YES'],
                			-default  => $lineIO{'callprogress'});

			unless ( -e "/etc/e-smith" ) {
				print SarkSubs::aLabelFor('openfirewall');
            			print $q->popup_menu (
                			-name     => 'openfirewall',
                        		-values   => ['NO', 'YES'],
                			-default  => $lineIO{'openfirewall'});

        			print SarkSubs::aLabelFor('host');
        			print $q->textfield (
                                	-name     => 'host',
                                	-override => 1,
                                	-style    => "color: $hashColorDefaults{host}",
                                	-default  => $lineIO{'host'});

        			print SarkSubs::aLabelFor('subnet1');
        			print $q->textfield (
                                	-name     => 'subnet1',
                                	-override => 1,
                                	-style    => "color: $hashColorDefaults{subnet1}",
                                	-default  => $lineIO{'subnet1'});

        			print SarkSubs::aLabelFor('subnet2');
        			print $q->textfield (
                                	-name     => 'subnet2',
                                	-override => 1,
                                	-style    => "color: $hashColorDefaults{subnet2}",
                                	-default  => $lineIO{'subnet2'});
			}
         	}
         }

         if ($carriertype eq "Spa3xxx" ) {
               print SarkSubs::aLabelFor('macaddr');
               print $q->textfield (
                		-name     => 'macaddr',
                		-override => 1,
                                -style    => "color: $hashColorDefaults{macaddr}",
                		-default  => $lineIO{'macaddr'},
        			-size     => 10);
         }
         if ($carriertype eq "group") {
               print SarkSubs::aLabelFor('predial');
               print $q->textfield (
                                -name     => 'predial',
                                -override => 1,
                                -style    => "color: $hashColorDefaults{predial}",
                                -default  => $lineIO{'predial'},
                                -size     => 20);

               print SarkSubs::aLabelFor('postdial');
               print $q->textfield (
                                -name     => 'postdial',
                                -override => 1,
                                -style    => "color: $hashColorDefaults{postdial}",
                                -default  => $lineIO{'postdial'},
                                -size     => 20);

		print SarkSubs::aLabelFor('callprogress');
            	print $q->popup_menu (
                		-name     => 'callprogress',
                       		-values   => ['NO', 'YES'],
                		-default  => $lineIO{'callprogress'});
         }
#
#       TAB DIVEND
#
        print '</div>';
#    }

#
#   TAB Peer
#
    if ($technology eq 'SIP' || $technology =~ 'IAX2' || $lineIO{'carrier'} =~ /SPA-3\d{3}FXO/) {
    	print '<div id="peer" class="ui-tabs-hide">';

#    	print '<div class="buttons">';
#    		SarkSubs::sark_button_save("save");
#   		unless ($technology eq 'Analogue' || $technology eq 'ISDNBRI' || $technology eq 'DAHDI' || $technology eq 'Zap')  {
#    			SarkSubs::sark_button_delete("delete");
#    		}
#    		SarkSubs::sark_button_cancel();
#    		SarkSubs::sark_button_commit( "commit", $commit );
#    	print '</div>';
        print SarkSubs::aLabelFor('peername');
        print $q->textfield (
                                -name     => 'peername',
                                -override => 1,
                                -style    => "color: $hashColorDefaults{peername}",
                                -default  => $lineIO{'peername'},
                                -size     => 20);
#

        if ($technology eq 'SIP' || $technology =~ 'IAX2' || $lineIO{'carrier'} =~ /SPA-3\d{3}FXO/) {
#               print SarkSubs::aLabelFor('sipiaxpeer');
               print $q->textarea (
               			-class    => 'sipiax',
                                -name     => 'sipiaxpeer',
                                -rows 	  => 15,
                                -columns  => 100,
                                -default  => $lineIO{'sipiaxpeer'});
        }
#
#       TAB DIVEND
#
        print '</div>';
    }

#    if ($epage == 4) {
#
#       TAB User
    if ($technology eq 'SIP' || $technology =~ 'IAX2' || $lineIO{'carrier'} =~ /SPA-3\d{3}FXO/) {
    	print '<div id="user" class="ui-tabs-hide">';

#    	print '<div class="buttons">';
#    		SarkSubs::sark_button_save("save");
#   		unless ($technology eq 'Analogue' || $technology eq 'ISDNBRI' || $technology eq 'DAHDI' || $technology eq 'Zap')  {
#    			SarkSubs::sark_button_delete("delete");
#    		}
#    		SarkSubs::sark_button_cancel();
#    		SarkSubs::sark_button_commit( "commit", $commit );
#    	print '</div>';
        print SarkSubs::aLabelFor('sipiaxusername');
        print $q->textfield (
                                -name     => 'desc',
                                -override => 1,
                                -style    => "color: $hashColorDefaults{peername}",
                                -default  => $lineIO{'desc'},
                                -size     => 20);
#        print SarkSubs::aLabelFor('sipiaxuser');
        print $q->textarea (
        		        -class    => 'sipiax',
                                -name     => 'sipiaxuser',
                                -rows 	  => 15,
                                -columns  => 100,
                                -default  => $lineIO{'sipiaxuser'});
         if ($lineIO{'carrier'} =~ /SPA-3\d{3}FXO/) {
#               print SarkSubs::aLabelFor('provision');
               print $q->textarea (
               	                -class    => 'sipiax',
                                -name     => 'provision',
                                -rows 	  => 15,
                                -columns  => 100,
                                -default  => $lineIO{'provision'});
         }
#
#       TAB DIVEND
#
        print '</div>';
    }
#
#  end of TABS DIV
#
    print '</div>';

    SarkSubs::sark_footer($q);
#    SarkSubs::SQLiteDisconnect($dbh);
}

sub performGetKey ($)
{
    my $q = shift;
    my $msg;
    my $dbh = SarkSubs::SQLiteConnect();
    my @Carriers = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT pkey FROM Carrier WHERE technology!='DAHDI'"));
    SarkSubs::SQLiteDisconnect($dbh);
#    SarkSubs::sark_header ($q, $msg, '', SarkSubs::tt_display('Sarktrunkh23'));
    my @displayCarriers = ();
    foreach (@Carriers) {
	unless (/Analog/ || /^ISDN/ || /Zap/ ) {
      		push (@displayCarriers, $_);
    	}
    }
    unless (@Carriers) {
    	push (@displayCarriers, "No Carriers");
    }

    SarkSubs::sark_header ($q);
    print h1( SarkSubs::tt_display('Sarksystemh11').'Trunk' );
    print '<hr class="hr" />';
    print '<div class="buttons">';
    SarkSubs::sark_button_cancel();
    SarkSubs::sark_button_ok("continue");
    print '</div>';


    print '<div>';
#    print "<div class=\"notes\">
#        <h4>Carrier Select</h4>
#        <p class=\"last\">Choose the carrier or trunk type you
#        wish to use from the drop down, then click on the blue continue button...</p>
#       </div>";



    print SarkSubs::aLabelFor('carrier');
    print $q->popup_menu (
    	  	-name    => 'carrier',
               	-values  => [@displayCarriers]);

    print '</div>';
    SarkSubs::sark_footer($q);
}


sub performCreate ($)
{
    my $q = shift;
#    my $line     = $q->param ('line');
    my $carrier  = $q->param ('carrier');
#    print STDERR "CARRIER => $carrier\n";
    %hashScrnVars = $q->Vars;
    my $dbh = SarkSubs::SQLiteConnect();
    my $carriertype 	= SarkSubs::SQLiteGet( $dbh, "SELECT carriertype FROM Carrier where pkey = '$carrier'" );
    my $technology      = SarkSubs::SQLiteGet( $dbh, "SELECT technology FROM Carrier where pkey = '$carrier'" );
    my $host	  	= SarkSubs::SQLiteGet( $dbh, "SELECT host FROM Carrier where pkey = '$carrier'" );
    my $commit	  	= SarkSubs::SQLiteGet( $dbh, "SELECT MYCOMMIT FROM globals where pkey = 'global'" );
    SarkSubs::SQLiteDisconnect($dbh);

#    my $msg;

#    SarkSubs::sark_header ($q, '', 'Trunk Definitions', 'Add VOIP, or SIP/Analog Gateway Trunk');
#    SarkSubs::sark_header ($q, $msg, '', SarkSubs::tt_display('Sarktrunkh24'));
    SarkSubs::sark_header ($q);
    print h1( SarkSubs::tt_display('Sarksystemh11')."Trunk Create: Carriertype is $carriertype" );
     print '<hr class="hr" />';
    if (@Verrors) {
	SarkSubs::sark_error(\@Verrors);
    }
    print '<div class="buttons">';
        SarkSubs::sark_button_cancel();
    	if ($carriertype eq 'DiD') {
    		SarkSubs::sark_button_saveDiD();
    	}
    	else {
    		SarkSubs::sark_button_save("save");
    	}

    	SarkSubs::sark_button_commit("latecommit",$commit);  # this one should be red !
    print "</div>";

    print "<div>";
#    print "<div class=\"notes\">
#        <h4>Trunk Details</h4>
#        <p class=\"last\">Fill out the details for the trunk type you've selected...</p>
#       </div>";

    if ($carriertype eq 'Sibling') {

 	print SarkSubs::aLabelFor('s-peername');
    	print $q->textfield (
        	-name     => 'peername',
        	-override => 1,
                -style    => "color: $hashColorDefaults{peername}",
        	-default  => $hashScrnVars{peername},
        	-size     => 20);
 	print SarkSubs::aLabelFor('host');
    	print $q->textfield (
        	-name     => 'host',
        	-override => 1,
                -style    => "color: $hashColorDefaults{host}",
        	-default  =>$hashScrnVars{host},
        	-size     => 20);

	unless ( -e "/etc/e-smith" ) {
		print SarkSubs::aLabelFor('openfirewall');
        	print $q->popup_menu (
        	-name     => 'openfirewall',
        	-values   => ['NO', 'YES'],
        	-default  => $hashScrnVars{'openfirewall'});
	}

        print SarkSubs::aLabelFor('password');
    	print $q->textfield (
                -name    => 'password',
                -override => 1,
                -style    => "color: $hashColorDefaults{host}",
                -default => $hashScrnVars{password},
                -size     => 5);
 	print SarkSubs::aLabelFor('privileged');
    	print $q->popup_menu (
                -name    => 'privileged',
                -values  => ['YES','NO'],
                -default => $hashScrnVars{privileged});
    }
    elsif ($carriertype eq 'Spa3xxx') {
 	print SarkSubs::aLabelFor('peername');
    	print $q->textfield (
        	-name     => 'peername',
                -override => 1,
		-readonly => 'readonly',
                -style    =>'background-color: lightgrey',
                -default  => $hashScrnVars{peername},
        	-size     => 20);
 	print SarkSubs::aLabelFor('remotenum');
    	print $q->textfield (
                -name     => 'ddi',
                -override => 1,
                -style    => "color: $hashColorDefaults{ddi}",
                -default  => $hashScrnVars{ddi},
        	-size     => 20);
 	print SarkSubs::aLabelFor('macaddr');
    	print $q->textfield (
                -name     => 'macaddr',
                -override => 1,
                -style    => "color: $hashColorDefaults{macaddr}",
                -default  => $hashScrnVars{macaddr},
        	-size     => 10);

 	print SarkSubs::aLabelFor('desc');
    	print $q->textfield (
                 -name     => 'desc',
                 -override => 1,
                 -style    => "color: $hashColorDefaults{desc}",
                 -default  => $hashScrnVars{desc},
        	 -size     => 20);
     }

     elsif ($carriertype eq 'group' || $carriertype eq 'Custom') {

 	print SarkSubs::aLabelFor('peername');
    	print $q->textfield (
                -name     => 'peername',
                -override => 1,
                -style    => "color: $hashColorDefaults{peername}",
                -default  => $hashScrnVars{peername},
        	-size     => 20);
 	print SarkSubs::aLabelFor('predial');
    	print $q->textfield (
                -name     => 'predial',
                -override => 1,
                -style    => "color: $hashColorDefaults{predial}",
                -default  =>$hashScrnVars{predial},
        	-size     => 20);
 	print SarkSubs::aLabelFor('postdial');
    	print $q->textfield (
                -name     => 'postdial',
                -override => 1,
                -style    => "color: $hashColorDefaults{postdial}",
                -default  => $hashScrnVars{postdial},
        	-size     => 20);
    print SarkSubs::aLabelFor('routeable');
    	print $q->popup_menu (
                 -name    => 'routeable',
                 -values  => ['NO','YES'],
                 -default => $hashScrnVars{routeable});

#        print SarkSubs::aLabelFor('desc');
#    	print $q->textfield (
#                 -name     => 'desc',
#                 -override => 1,
#                 -style    => "color: $hashColorDefaults{desc}",
#                 -default  => $hashScrnVars{desc},
#        	 -size     => 20);
    }
    elsif ($carriertype eq 'DiD') {
 	print SarkSubs::aLabelFor('didstart');
    	print $q->textfield (
                 -name     => 'didstart',
                 -override => 1,
                 -style    => "color: $hashColorDefaults{didstart}",
                 -default  => $hashScrnVars{didstart},
        	 -size     => 20);
 	print SarkSubs::aLabelFor('didend');
    	print $q->textfield (
                 -name     => 'didend',
                 -override => 1,
                 -style    => "color: $hashColorDefaults{didend}",
                 -default  => $hashScrnVars{didend},
        	 -size     => 20);
 	print SarkSubs::aLabelFor('inprefix');
    	print $q->textfield (
                 -name     => 'inprefix',
                 -override => 1,
                 -style    => "color: $hashColorDefaults{inprefix}",
                 -default  => $hashScrnVars{inprefix},
        	 -size     => 20);
 	print SarkSubs::aLabelFor('alertinfo');
    	print $q->textfield (
                 -name     => 'alertinfo',
                 -override => 1,
                 -style    => "color: $hashColorDefaults{alertinfo}",
                 -default  => $hashScrnVars{alertinfo},
        	 -size     => 20);
 	print SarkSubs::aLabelFor('smartlink');
    	print $q->popup_menu (
                 -name    => 'smartlink',
                 -values  => ['NO','YES'],
                 -default => $hashScrnVars{smartlink});
# 	print SarkSubs::aLabelFor('desc');
#    	print $q->textfield (
#                 -name     => 'desc',
#                 -override => 1,
#                 -style    => "color: $hashColorDefaults{desc}",
#                 -default  => $hashScrnVars{desc},
#        	 -size     => 20);
    }

    elsif ($carriertype eq 'CLID') {
 	print SarkSubs::aLabelFor('clidstart');
    	print $q->textfield (
                 -name     => 'clidnumber',
                 -override => 1,
                 -style    => "color: $hashColorDefaults{clidnumber}",
                 -default  => $hashScrnVars{clidnumber},
        	 -size     => 20);
# 	print SarkSubs::aLabelFor('desc');
#    	print $q->textfield (
#                 -name     => 'desc',
#                 -override => 1,
#                 -style    => "color: $hashColorDefaults{desc}",
#                 -default  => $hashScrnVars{desc},
#        	 -size     => 20);
    }

    elsif ($carriertype eq 'Class')  {
 	print SarkSubs::aLabelFor('classmask');
    	print $q->textfield (
                 -name     => 'classmask',
                 -override => 1,

                 -style    => "color: $hashColorDefaults{classmask}",
                 -default  =>$hashScrnVars{classmask},
        	 -size     => 20);

# 	print SarkSubs::aLabelFor('desc');
#    	print $q->textfield (
#                 -name     => 'desc',
#                 -override => 1,
#                 -style    => "color: $hashColorDefaults{desc}",
#                 -default  => $hashScrnVars{desc},
#        	 -size     => 20);
     }
     else {
 	print SarkSubs::aLabelFor('remotenum');
    	print $q->textfield (
                 -name     => 'didnumber',
                 -override => 1,
                 -style    => "color: $hashColorDefaults{didnumber}",
                 -default  => $hashScrnVars{didnumber},
        	 -size     => 20);

 	print SarkSubs::aLabelFor('host');
    	print $q->textfield (
                 -name     => 'host',
                 -override => 1,
                 -style    => "color: $hashColorDefaults{host}",
                 -default  => $hashScrnVars{host},
        	 -size     => 20);
	unless ( -e "/etc/e-smith" ) {
		print SarkSubs::aLabelFor('openfirewall');
        	print $q->popup_menu (
        	-name     => 'openfirewall',
        	-values   => ['NO', 'YES'],
        	-default  => $hashScrnVars{'openfirewall'});
	}
 	print SarkSubs::aLabelFor('username');
    	print $q->textfield (
                 -name     => 'username',
                 -override => 1,
                 -style    => "color: $hashColorDefaults{username}",
                 -default  => $hashScrnVars{username},
        	 -size     => 20);

 	print SarkSubs::aLabelFor('password');
    	print $q->textfield (
                 -name     => 'password',
                 -override => 1,
                 -style    => "color: $hashColorDefaults{password}",
                 -default  => $hashScrnVars{password},
        	 -size     => 20);

#        print SarkSubs::aLabelFor('desc');
#    	print $q->textfield (
#                 -name     => 'desc',
#                 -override => 1,
#                 -style    => "color: $hashColorDefaults{desc}",
#                 -default  => $hashScrnVars{desc},
#        	 -size     => 20);
        print SarkSubs::aLabelFor('register');
    	print $q->popup_menu (
                 -name    => 'regthistrunk',
                 -values  => ['NO','YES']);
#                 -default => $hashScrnVars{regthistrunk});
     }


    print SarkSubs::aLabelFor('trunkname');
    print $q->textfield (
                 -name     => 'trunkname',
                 -override => 1,
                 -style    => "color: $hashColorDefaults{trunkname}",
                 -default  => $hashScrnVars{trunkname},
        	 -size     => 20);
    print "</div>";

    print $q->hidden (-name => 'carrier',  -override => 1, -default => $carrier);
    print $q->hidden (-name => 'carriertype',  -override => 1, -default => $carriertype);
    print $q->hidden (-name => 'technology',  -override => 1, -default => $technology);

#    print STDERR "CARRIERSTUFF = $carrier, $carriertype, $technology \n";
    SarkSubs::sark_footer($q);
}

sub createSibling ($) {
#	print STDERR "IN SIBLING\n";
	my $q = shift;
	my $fwc;
        %hashScrnVars = $q->Vars;
#        %hashVarDefaults = (
#        	"host" 		=> undef,
#		"peername" 	=> undef,
#                "carrier"	=> undef,
#                "username"      => undef,
#		"privileged" 	=> undef,
#                "desc" 		=> undef,
#                "sipiaxpeer"  	=> undef,
#                "sipiaxuser"  	=> undef,
#        );
        if ($hashScrnVars{technology} eq 'IAX2') {
    		if ($hashScrnVars{privileged} eq 'YES') {
			$hashScrnVars{username} 	= $systemName.$hashScrnVars{peername};
                        $hashScrnVars{pkey}             = $hashScrnVars{peername}.$systemName;
                        $hashScrnVars{desc}             = $hashScrnVars{pkey};
			$hashScrnVars{peername} 	= $hashScrnVars{pkey};

#                       $hashScrnVars{username} 	= $systemName.$hashScrnVars{peername};
#			$hashScrnVars{pkey}    		= $hashScrnVars{peername}.$systemName;
#        		$hashScrnVars{peername} 	= $hashScrnVars{pkey};
        	}
        	else {
			$hashScrnVars{username} 	= $systemName."\~".$hashScrnVars{peername};
			$hashScrnVars{pkey}    		= $hashScrnVars{peername}."\~".$systemName;
            $hashScrnVars{desc}         = $hashScrnVars{pkey};
        	$hashScrnVars{peername} 	= $hashScrnVars{pkey};
		}
		$hashScrnVars{subnetstr} = '';
		$hashScrnVars{subnet1str} = '';
		$hashScrnVars{subnet2str} = '';
        $hashScrnVars{remotenum} = $hashScrnVars{peername};
		if ( $hashScrnVars{openfirewall} eq "YES" ) {
			$hashScrnVars{subnetstr} = "ACCEPT net:$hashScrnVars{host} \$FW udp 4569 #TRUNK$hashScrnVars{pkey}";
			$fwc = 1;
		}

		if ( $hashScrnVars{subnet1} ) {
			$hashScrnVars{subnet1str} = "ACCEPT net:$hashScrnVars{subnet1} \$FW udp 4569 #TRUNK$hashScrnVars{pkey}";
			$fwc = 1;
		}
		if ( $hashScrnVars{subnet2} ) {
			$hashScrnVars{subnet2str} = "ACCEPT net:$hashScrnVars{subnet2} \$FW udp 4569 #TRUNK$hashScrnVars{pkey}";
			$fwc = 1;
		}

        }
        if ($hashScrnVars{technology} eq 'SIP' ) {
        	if ($hashScrnVars{privileged} eq 'YES') {
 			$hashScrnVars{username} 	= $hashScrnVars{peername};
 			$hashScrnVars{pkey} 		= $hashScrnVars{peername};
                        $hashScrnVars{desc} 		= $hashScrnVars{username};
     		}

        	else {
                	if ($hashScrnVars{carrier} eq 'SIP-to-SIP') {
        			$hashScrnVars{username} 	= $hashScrnVars{peername}."~";
                                $hashScrnVars{desc} 		= $hashScrnVars{username};
 				$hashScrnVars{pkey}   	  	= $hashScrnVars{peername}."~";
                        }
                        else {
                                $hashScrnVars{username} 	= $hashScrnVars{peername};
                                $hashScrnVars{desc} 		= $hashScrnVars{username};
 				$hashScrnVars{pkey}   		= $hashScrnVars{peername};
                     	}
                }
        }
        copyTemplates();
        createValidate();
        unless ($gonogo) {
        	insertRow ();
		if ($fwc) {
        		createEnd($q,"Entry Created! - RESTART FIREWALL!");
		}
		else {
			createEnd($q,"Entry Created!");
		}
        }
        else {
        	performCreate ($q);
        }
}

sub createGroup ($) {
#print STDERR "IN GROUP\n";
	my $q = shift;
        %hashScrnVars = $q->Vars;
#        %hashVarDefaults = (
#		"peername" 	=> undef,
#                "carrier"	=> undef,
#                "predial"	=> undef,
#		"postdial" 	=> undef,
#                "desc"        	=> undef,
#        );
        $hashScrnVars{pkey} = $hashScrnVars{peername};
        createValidate();
        unless ($gonogo) {
        	insertRow ();
        	createEnd($q,"Entry Created!");
        }
        else {
        	performCreate ($q);
        }
}

sub createDid ($) {
#print STDERR "IN DiD\n";
	my $q = shift;
        my $dbh;
        my $extlen;
        %hashScrnVars = $q->Vars;

        if ($hashScrnVars{smartlink} eq "YES") {
                $dbh = SarkSubs::SQLiteConnect();
        	$extlen = SarkSubs::SQLiteGet($dbh, "SELECT EXTLEN FROM globals where pkey = 'global'") || 4;
                SarkSubs::SQLiteDisconnect($dbh);
        }

#        if ( $hashScrnVars{didend} ) {
#               	unless ( $hashScrnVars{didend} =~ /^\d+$/ ) {
##       		$hashScrnVars{didend} = $hashScrnVars{didstart};
#                        push ( @Verrors, "Second key is not numeric");
#                        $hashColorDefaults{didend} = 'red';
#                        performCreate ($q);
#                        exit 0;
#                }
#        }

        if ( $hashScrnVars{didstart} =~ /^\d+$/ ) {
                if ( $hashScrnVars{didend} ) {
                        unless ( $hashScrnVars{didend} =~ /^\d+$/ ) {
#       			$hashScrnVars{didend} = $hashScrnVars{didstart};
                        	push ( @Verrors, "Second key is not numeric");
                        	$hashColorDefaults{didend} = 'red';
                        	performCreate ($q);
                        	exit 0;
                	}
	       		unless ( $hashScrnVars{didstart} le $hashScrnVars{didend} ) {
        	        	push ( @Verrors, "Second key is less than first");
                		$hashColorDefaults{didstart} = 'red';
                        	$hashColorDefaults{didend} = 'red';
                		performCreate ($q);
                		exit 0;
                	}
		}
                else {
                	$hashScrnVars{didend} = $hashScrnVars{didstart};
                }
        }
        else {
        	if ( $hashScrnVars{didend} ) {
                        push ( @Verrors, "Second key not allowed if first key is alphanumeric");
                	$hashColorDefaults{didstart} = 'red';
                        $hashColorDefaults{didend} = 'red';
                	performCreate ($q);
                	exit 0;
                }
                else {
                	$hashScrnVars{didend} = $hashScrnVars{didstart};
                }
        }

        my $didstart = $hashScrnVars{didstart};
        $hashScrnVars{pkey} = $didstart;
        createValidate();
        unless ($gonogo) {
        	while ( $didstart le $hashScrnVars{didend} ) {
#
#		cheat by modifying just the key in the OutBuf
#
        		$hashVarOutBuf{pkey} = $didstart;
#
#		Smartlink
#
                        if ($hashScrnVars{smartlink} eq "YES") {
                        	$dbh = SarkSubs::SQLiteConnect();
                        	my $tkey = substr $hashVarOutBuf{pkey}, -$extlen;
                                if (SarkSubs::SQLiteGet($dbh, "SELECT pkey FROM IPphone where pkey = '$tkey'") ) {
                                	$hashVarOutBuf{openroute} = $tkey;
                                    $hashVarOutBuf{closeroute} = $tkey;
                                    $hashVarOutBuf{routeclassopen} = "1";
                                    $hashVarOutBuf{routeclassclosed} = "1";
                                }
                                else {
                                    $hashVarOutBuf{openroute} = "Operator";
                                    $hashVarOutBuf{closeroute} = "Operator";
                                    $hashVarOutBuf{routeclassopen} = "100";
                                    $hashVarOutBuf{routeclassclosed} = "100";
                                }
                        	SarkSubs::SQLiteDisconnect($dbh);
                        }
                        insertRow();
                	$didstart++;
        	}
        	createEnd($q,"Entries Created!");
        }
        else {
                performCreate ($q);
        }
}

sub createVoip ($) {
	my $q = shift;
	my $fwc;
        %hashScrnVars = $q->Vars;

    $hashScrnVars{pkey} = $hashScrnVars{didnumber};
    my $range = 10000;
    $hashScrnVars{peername} = "peer".int(rand($range));
    if ($hashScrnVars{technology} eq 'IAX2') {
		$hashScrnVars{desc} = $hashScrnVars{username};
	}
	else {
		$hashScrnVars{desc} = "user".int(rand($range));
	}
	

	$hashScrnVars{subnetstr} = '';
	$hashScrnVars{subnet1str} = '';
	$hashScrnVars{subnet2str} = '';

        if ( $hashScrnVars{regthistrunk} eq "YES" ) {
        	$hashScrnVars{register} = "$hashScrnVars{username}:$hashScrnVars{password}\@$hashScrnVars{host}/$hashScrnVars{username}";
        }
#	print STDERR "openfw IS $hashScrnVars{openfirewall} \n";
	if ( $hashScrnVars{openfirewall} eq "YES" ) {
		$fwc = 1;
		if ($hashScrnVars{technology} eq 'IAX2') {
			$hashScrnVars{subnetstr} = "ACCEPT net:$hashScrnVars{host} \$FW udp 4569 #TRUNK$hashScrnVars{pkey}";
		}
		else {
			$hashScrnVars{subnetstr} = "ACCEPT net:$hashScrnVars{host} \$FW udp 5060,10000:20000 #TRUNK$hashScrnVars{pkey}";
		}
	}
	if ( $hashScrnVars{subnet1} ) {
		$fwc = 1;
		if ($hashScrnVars{technology} eq 'IAX2') {
			$hashScrnVars{subnet1str} = "ACCEPT net:$hashScrnVars{subnet1} \$FW udp 4569 #TRUNK$hashScrnVars{pkey}";
		}
		else {
			$hashScrnVars{subnet1str} = "ACCEPT net:$hashScrnVars{subnet1} \$FW udp 5060,10000:20000 #TRUNK$hashScrnVars{pkey}";
		}
	}
	if ( $hashScrnVars{subnet2} ) {
		$fwc = 1;
		if ($hashScrnVars{technology} eq 'IAX2') {
			$hashScrnVars{subnet2str} = "ACCEPT net:$hashScrnVars{subnet2} \$FW udp 4569 #TRUNK$hashScrnVars{pkey}";
		}
		else {
			$hashScrnVars{subnet2str} = "ACCEPT net:$hashScrnVars{subnet2} \$FW udp 5060,10000:20000 #TRUNK$hashScrnVars{pkey}";
		}
	}
        copyTemplates ();
        createValidate();
        unless ($gonogo) {
        	insertRow ();
        	if ($fwc) {
        		createEnd($q,"Entry Created! - RESTART FIREWALL!");
		}
		else {
			createEnd($q,"Entry Created!");
		}
        }
        else {
        	performCreate ($q);
        }
}

sub createSpa ($) {
#print STDERR "IN SPA\n";
	my $q = shift;
        %hashScrnVars = $q->Vars;

        $hashScrnVars{username}		= $hashScrnVars{didnumber};
	$hashScrnVars{pkey}     	= $hashScrnVars{didnumber};
        $hashScrnVars{host}     	= 'dynamic';

        copyTemplates ();
        createValidate();
        unless ($gonogo) {
        	insertRow ();
        	createEnd($q,"Entry Created!");
        }
        else {
        	performCreate ($q);
        }
}

sub createClid ($) {
#print STDERR "IN CLID\n";
	my $q = shift;
        %hashScrnVars = $q->Vars;

        $hashScrnVars{pkey} = $hashScrnVars{clidnumber};
        createValidate();
        unless ($gonogo) {
        	insertRow ();
        	createEnd($q,"Entry Created!");
        }
        else {
        	performCreate ($q);
        }
}

sub createClass ($) {
#print STDERR "IN CLASS\n";
	my $q = shift;
        %hashScrnVars = $q->Vars;

        $hashScrnVars{pkey} = $hashScrnVars{classmask};
        createValidate();
        unless ($gonogo) {
        	insertRow ();
        	createEnd($q,"Entry Created!");
        }
        else {
        	performCreate ($q);
        }
}

sub createValidate () {
#	$hashVarDefaults{pkey} = undef;
        my $dbh = SarkSubs::SQLiteConnect();
        my %hashRecordColumns = SarkSubs::SQLiteGetHashCols($dbh, 'lineIO');
        SarkSubs::SQLiteDisconnect($dbh);
        $gonogo = SarkSubs::validateTuple(\%hashScrnVars, \%hashRecordColumns, \%hashVarRegexs,
    			\%hashVarOutBuf, \%hashColorDefaults,
                        \@Verrors);
}

sub copyTemplates () {
#print STDERR "IN TEMPLATES\n";
        my $dbh = SarkSubs::SQLiteConnect();
    	$hashScrnVars{sipiaxuser} = SarkSubs::SQLiteGet( $dbh, "SELECT sipiaxuser FROM Carrier where pkey = '$hashScrnVars{carrier}'" );
		$hashScrnVars{sipiaxpeer} = SarkSubs::SQLiteGet( $dbh, "SELECT sipiaxpeer FROM Carrier where pkey = '$hashScrnVars{carrier}'" );
        $hashScrnVars{provision} = SarkSubs::SQLiteGet( $dbh, "SELECT provision FROM Carrier where pkey = '$hashScrnVars{carrier}'" );
#        $hashScrnVars{register} = SarkSubs::SQLiteGet( $dbh, "SELECT register FROM Carrier where pkey = '$hashScrnVars{carrier}'" );
    	SarkSubs::SQLiteDisconnect($dbh);
#        print STDERR "Carrier is $hashScrnVars{carrier}\n";
        if (exists $hashScrnVars{sipiaxpeer} ) {
      		$hashScrnVars{sipiaxpeer} =~ s/username=/username=$hashScrnVars{username}/;
      		$hashScrnVars{sipiaxpeer} =~ s/fromuser=/fromuser=$hashScrnVars{username}/;
      		$hashScrnVars{sipiaxpeer} =~ s/secret=/secret=$hashScrnVars{password}/;
      		$hashScrnVars{sipiaxpeer} =~ s/host=/host=$hashScrnVars{host}/;
        	$hashScrnVars{sipiaxpeer} =~ s/^\s+//;
        	$hashScrnVars{sipiaxpeer} =~ s/\s+$//;
            if ( exists $hashScrnVars{privileged} && $hashScrnVars{privileged} eq 'YES' ) {
				$hashScrnVars{sipiaxpeer} =~ s/mainmenu/priv_sibling/;
            }  
            unless ($hashScrnVars{sipiaxpeer} =~ /allow=/ ) {
        		$hashScrnVars{sipiaxpeer} .= "\ndisallow=all\nallow=alaw\nallow=ulaw";
        	}
        }
#        print STDERR "peer is $hashScrnVars{sipiaxpeer}\n";
        if (exists $hashScrnVars{sipiaxuser} ) {
      		$hashScrnVars{sipiaxuser} =~ s/username=/username=$hashScrnVars{username}/;
      		$hashScrnVars{sipiaxuser} =~ s/fromuser=/fromuser=$hashScrnVars{username}/;
      		$hashScrnVars{sipiaxuser} =~ s/secret=/secret=$hashScrnVars{password}/;
        	$hashScrnVars{sipiaxuser} =~ s/^\s+//;
        	$hashScrnVars{sipiaxuser} =~ s/\s+$//;
        	if ( exists $hashScrnVars{privileged} && $hashScrnVars{privileged} eq 'NO' ) {
				$hashScrnVars{sipiaxuser} =~ s/context=internal/context=mainmenu/;
            }
        }

        if ( exists $hashScrnVars{provision}) {
#        $hashScrnVars{provision} =~ s/\$ipaddr/$ipaddr/g;
        $hashScrnVars{provision} =~ s/\$line/$hashScrnVars{pkey}/g;
#        $hashScrnVars{provision} =~ s/\$password/$password/;
	}
}

sub insertRow() {

    unless ($gonogo) {
#
#	Set defaults into the output buffer
#
        while ( my ($key, $value) = each %hashVarDefaults ) {
               	unless ( $hashVarOutBuf{$key} ) {
                       	if ($hashVarDefaults{$key}) {
                       		$hashVarOutBuf{$key} = $hashVarDefaults{$key};
                        }
                }
        }
#
#	Create record
#
    	my $dbh = SarkSubs::SQLiteConnect();
        SarkSubs::newTuple( \%hashVarOutBuf, "lineIO", $dbh);
        SarkSubs::SQLiteDisconnect($dbh);
    }
}

sub createEnd($$) {
    my $q = shift;
    my $response = shift;
    if ($q->param('latecommit.x') ) {
                SarkSubs::sysCommit($q);
    }
    showInitial ($q,$response);
}

sub performLinesIn ($)
{
    my $q = shift;
    my %hashScrnVars = $q->Vars ;
    my $response;
    my $range = 10000;
    my $fwc;
    my $dbh = SarkSubs::SQLiteConnect();
    my %lineIO = SarkSubs::SQLiteGetHash($dbh, "SELECT * FROM lineIO WHERE pkey = '$hashScrnVars{pkey}'");

#   call subs to set the routeclass
    if ($hashScrnVars{openroute}) {
    	$hashScrnVars{routeclassopen} = SarkSubs::routeClass($hashScrnVars{openroute});
    }
    if  ($hashScrnVars{closeroute}) {
    $hashScrnVars{routeclassclosed} = SarkSubs::routeClass($hashScrnVars{closeroute});
    }
    unless ($hashScrnVars{peername}) {
    	$hashScrnVars{peername} = "peer".int(rand($range));
    }
    unless ($hashScrnVars{desc}) {
        $hashScrnVars{desc} = "user".int(rand($range));
    }

#   check firewall rules
	$hashScrnVars{subnetstr} = '';
	$hashScrnVars{subnet1str} = '';
	$hashScrnVars{subnet2str} = '';

#	print STDERR "openfw IS $hashScrnVars{openfirewall} \n";
	if ( $hashScrnVars{openfirewall} eq "YES" && $hashScrnVars{host} ) {
		$fwc = 1;
		if ($lineIO{technology} eq 'IAX2') {
			$hashScrnVars{subnetstr} = "ACCEPT net:$hashScrnVars{host} \$FW udp 4569 #TRUNK$hashScrnVars{pkey}";
		}
		else {
			$hashScrnVars{subnetstr} = "ACCEPT net:$hashScrnVars{host} \$FW udp 5060,10000:20000 #TRUNK$hashScrnVars{pkey}";
		}
	}
	if ( $hashScrnVars{subnet1} ) {
		$fwc = 1;
		if ($lineIO{technology} eq 'IAX2') {
			$hashScrnVars{subnet1str} = "ACCEPT net:$hashScrnVars{subnet1} \$FW udp 4569 #TRUNK$hashScrnVars{pkey}";
		}
		else {
			$hashScrnVars{subnet1str} = "ACCEPT net:$hashScrnVars{subnet1} \$FW udp 5060,10000:20000 #TRUNK$hashScrnVars{pkey}";
		}
	}
	if ( $hashScrnVars{subnet2} ) {
		$fwc = 1;
		if ($lineIO{technology} eq 'IAX2') {
			$hashScrnVars{subnet2str} = "ACCEPT net:$hashScrnVars{subnet2} \$FW udp 4569 #TRUNK$hashScrnVars{pkey}";
		}
		else {
			$hashScrnVars{subnet2str} = "ACCEPT net:$hashScrnVars{subnet2} \$FW udp 5060,10000:20000 #TRUNK$hashScrnVars{pkey}";
		}
	}

#   update the row

    $gonogo = SarkSubs::validateTuple(\%hashScrnVars, \%lineIO, \%hashVarRegexs,
    			\%hashVarUpdBuf, \%hashColorDefaults,
                        \@Verrors);
    $hashScrnVars{sipiaxpeer} =~ s/\r//g;
    $hashScrnVars{sipiaxuser} =~ s/\r//g;
    unless ( $gonogo ) {
    	SarkSubs::updateTuple (\%hashVarUpdBuf, \%lineIO, "lineIO", $dbh);
    }
    SarkSubs::SQLiteDisconnect($dbh);

    if ( $gonogo ) {
    	linesIn($q);
        return 0;
    }

    if ($q->param('commit.x')) {
        SarkSubs::sysCommit($q);
    }
    if ($fwc) {
    	$response .= "Entry Created! - RESTART FIREWALL!";
    }
    showInitial ($q,$response);
    return 0;

}

sub performDelete ($)
{
    my $q = shift;
    my $pkey = $q->param ('pkey');
#    if ($pkey eq "Analog-In" || $pkey =~ /PIKA/ ) {
	if ($pkey =~ /PIKA/ ) {
	showInitial ($q,"Delete of Analog/Digital trunk not allowed!");
	return 0;
    }
    my $dbh = SarkSubs::SQLiteConnect();
    my $macaddr = SarkSubs::SQLiteGet($dbh, "SELECT macaddr FROM lineIO where pkey = '$pkey'");
    SarkSubs::SQLiteDo($dbh, "UPDATE globals SET MYCOMMIT='YES' WHERE pkey='global'");
    SarkSubs::SQLiteDelete($dbh,"DELETE FROM lineIO where pkey='$pkey'");
    SarkSubs::SQLiteCommit($dbh);
    SarkSubs::SQLiteDisconnect($dbh);
    if ($macaddr) {
        my $lcmacaddr =  lc($macaddr);
        my @Provfiles = </tftpboot/*>;
        foreach (@Provfiles) {
        	if (  $_ =~ m{$lcmacaddr} || $_ =~ m{$macaddr} ) {
                	/(.*)/;
                	unlink $1;
        	}
        }
    }
#    $pkey =~ s/\*//;
    showInitial ($q, "<font color = red>Successfully deleted Trunk $pkey.</font color = red><font color = black>");
}
